@@ -3,13 +3,10 @@
 from __future__ import annotations
 
 from enum import Enum
-from typing import Protocol, Type, TypeVar
+from typing import Dict, List, Protocol, Type, TypeVar
 
 E = TypeVar("E", bound="AtlanError")
-RAISE_GITHUB_ISSUE = (
-    "Please raise an issue on the Java SDK GitHub repository providing context in which this error "
-    "occurred."
-)
+RAISE_GITHUB_ISSUE = "Please raise an issue on the Python SDK GitHub repository providing context in which this error occurred."
 
 
 class ErrorInfo(Protocol):
@@ -17,17 +14,52 @@ class ErrorInfo(Protocol):
     error_id: str
     error_message: str
     user_action: str
+    # Causes underlying the error
+    # if any (provided by the back-end)
+    error_causes: List[str]
+    # Unique identifier for the error
+    # (provided by the back-end)
+    backend_error_id: str
+    # A human-readable explanation
+    # of what caused the error
+    error_cause: str
+    # A URL linking to
+    # documentation that explains the error
+    error_doc: str
 
 
 class AtlanError(Exception):
-    def __init__(self, error_code: ErrorInfo, *args):
-        message = error_code.error_message.format(*args)
+    def __init__(self, error_code: ErrorInfo, *args, **kwargs):
+        try:
+            message = error_code.error_message.format(*args)
+        except (KeyError, ValueError):
+            message = error_code.error_message
         super().__init__(message)
-        self.code = error_code.error_id
-        self.status_code = error_code.http_error_code
+        self.error_code = error_code
+        self.error_code.backend_error_id = kwargs.get("backend_error_id", "")
+        self.error_code.error_cause = kwargs.get("error_cause", "")
+        self.error_code.error_doc = kwargs.get("error_doc", "")
 
     def __str__(self):
-        return f"{self.code or ''} {super().__str__()}"
+        return (
+            f"{self.error_code.error_id or ''}"
+            f"{' ' if self.error_code.error_id else ''}{super().__str__()}"
+            f"{' errorCause: ' + self.error_code.error_cause if self.error_code.error_cause else ''}"
+            f"{' errorDocument: ' + self.error_code.error_doc if self.error_code.error_doc else ''}"
+            f"{' (errorId: ' + self.error_code.backend_error_id + ')' if self.error_code.backend_error_id else ''}"
+            f" Suggestion: {self.error_code.user_action}"
+        )
+
+
+class DependencyNotFoundError(Exception):
+    """
+    Raised when a required external dependency is not installed.
+
+    This exception is typically used to indicate that an optional library
+    or plugin needed for a specific feature is missing.
+    """
+
+    pass
 
 
 class ApiConnectionError(AtlanError):
@@ -90,7 +122,7 @@ class ErrorCode(Enum):
     INVALID_REQUEST_PASSTHROUGH = (
         400,
         "ATLAN-PYTHON-400-000",
-        "Server responded with {0}: {1}",
+        "Server responded with an invalid request {0}: {1} -- caused by: {2}",
         "Check the details of the server's message to correct your request.",
         InvalidRequestError,
     )
@@ -182,14 +214,14 @@ class ErrorCode(Enum):
         400,
         "ATLAN-PYTHON-400-013",
         "Lineage was retrieved using hideProces=False. We do not provide a graph view in this case.",
-        "Retry your request for lineage setting hideProcess=true.",
+        "Retry your request for lineage setting hideProcess=True.",
         InvalidRequestError,
     )
     UNABLE_TO_TRANSLATE_FILTERS = (
         400,
         "ATLAN-PYTHON-400-014",
         "Unable to translate the provided include/exclude asset filters into JSON.",
-        "Verify the filters you provided. If the problem persists, please raise an issue on the Java SDK GitHub "
+        "Verify the filters you provided. If the problem persists, please raise an issue on the Python SDK GitHub "
         "repository providing context in which this error occurred.",
         InvalidRequestError,
     )
@@ -431,50 +463,197 @@ class ErrorCode(Enum):
     INVALID_PARAMETER_TYPE = (
         400,
         "ATLAN-PYTHON-400-048",
-        "Invalid parameter type for {0} should be {1}",
+        "Invalid parameter type for {0} should be {1}.",
         "Check that you have used the correct type of parameter.",
         InvalidRequestError,
     )
     GLOSSARY_MISSING_QUALIFIED_NAME = (
         400,
         "ATLAN-PYTHON-400-049",
-        "The qualified_name is not present in the Glossary",
-        "Check that the qualified_name is available from the Glossary",
+        "The qualified_name is not present in the Glossary.",
+        "Check that the qualified_name is available from the Glossary.",
         InvalidRequestError,
     )
     MISSING_OPTIONS = (
         400,
         "ATLAN-PYTHON-400-050",
-        "Options is not present in the AttributeDef",
-        "Please use the AttributeDef.create function to create the Options",
+        "Options is not present in the AttributeDef.",
+        "Please use the AttributeDef.create function to create the Options.",
         InvalidRequestError,
     )
     INVALID_PARAMETER_VALUE = (
         400,
         "ATLAN-PYTHON-400-051",
-        "{0} is an invalid value for {1} should be in {2}",
-        "Check that value you are using is valid",
+        "Invalid value(s) {0} provided for '{1}'. Expected one or more of the supported {2}.",
+        "Verify that each value corresponds to a valid SDK asset type.",
         InvalidRequestError,
     )
     ASSET_CAN_NOT_BE_ARCHIVED = (
         400,
         "ATLAN-PYTHON-400-052",
-        "Asset with guid: {0} is an asset of type {1} which does not support archiving",
-        "Please use purge if you wish to remove assets of this type",
+        "Asset with guid: {0} is an asset of type {1} which does not support archiving.",
+        "Please use purge if you wish to remove assets of this type.",
         InvalidRequestError,
     )
     METHOD_CAN_NOT_BE_INVOKED_ON_ASSET = (
         400,
         "ATLAN-PYTHON-400-053",
-        "This method cannot be invoked on the Asset class. Please invoke on a specific asset type",
-        "Please invoke this method on a sub-class of Asset",
+        "This method cannot be invoked on the Asset class. Please invoke on a specific asset type.",
+        "Please invoke this method on a sub-class of Asset.",
+        InvalidRequestError,
+    )
+    INVALID_CREDENTIALS = (
+        400,
+        "ATLAN-PYTHON-400-054",
+        "Credentials provided did not work: {0}.",
+        "Please double-check your credentials and test them again.",
+        InvalidRequestError,
+    )
+    MISSING_GLOSSARY_GUID = (
+        400,
+        "ATLAN-PYTHON-400-055",
+        "'glossary_guid' keyword argument is missing for asset type: {0}",
+        "Please double-check your method keyword arguments.",
+        InvalidRequestError,
+    )
+    MISSING_CREDENTIALS = (
+        400,
+        "ATLAN-PYTHON-400-056",
+        "Missing privileged credentials to impersonate users.",
+        "You must have both CLIENT_ID and CLIENT_SECRET configured to be able to impersonate users.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_TRANSLATE_ASSETS_DSL = (
+        400,
+        "ATLAN-PYTHON-400-057",
+        "Unable to construct the selected assets DSL JSON string for the data product.",
+        "Verify the assets index search request you provided."
+        + "If the problem persists, please raise an issue on the Python SDK GitHub "
+        + "repository providing context in which this error occurred.",
+        InvalidRequestError,
+    )
+    SSO_GROUP_MAPPING_ALREADY_EXISTS = (
+        400,
+        "ATLAN-PYTHON-400-058",
+        "SSO group mapping already exists between {0} (Atlan group) <-> {1} (SSO group).",
+        "You can use SSOClient.update_group_mapping() to update the existing group mapping.",
+        InvalidRequestError,
+    )
+    INVALID_UPLOAD_FILE_PATH = (
+        400,
+        "ATLAN-PYTHON-400-059",
+        "Unable to upload file, Error: {0}, Path: {1}.",
+        "Please check the provided file path for upload.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_DOWNLOAD_FILE = (
+        400,
+        "ATLAN-PYTHON-400-060",
+        "Unable to download file, Error: {0}, Path: {1}.",
+        "Please check the provided download file type and path.",
+        InvalidRequestError,
+    )
+    UNSUPPORTED_PRESIGNED_URL = (
+        400,
+        "ATLAN-PYTHON-400-061",
+        "Provided presigned URL's cloud provider storage is currently not supported for file uploads.",
+        "Please raise a feature request on the Python SDK GitHub to add support for it.",
+        InvalidRequestError,
+    )
+    INVALID_CONTRACT_JSON = (
+        400,
+        "ATLAN-PYTHON-400-062",
+        "Provided data contract JSON is invalid.",
+        "Please double-check your provided data contract JSON.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_RUN_BULK_WITH_SORTS = (
+        400,
+        "ATLAN-PYTHON-400-063",
+        "Unable to execute bulk search with user-defined sorting options.",
+        "Please ensure that no sorting options are included in your search request when performing a bulk search.",
+        InvalidRequestError,
+    )
+    OPENLINEAGE_NOT_CONFIGURED = (
+        400,
+        "ATLAN-PYTHON-400-064",
+        "Requested OpenLineage connector type '{0}' is not configured.",
+        "You must first run the appropriate marketplace package to "
+        + "configure OpenLineage for this connector before you can send events for it.",
+        InvalidRequestError,
+    )
+    MISSING_ID = (
+        400,
+        "ATLAN-PYTHON-400-065",
+        "No ID was provided when attempting to retrieve or update the object.",
+        "You must provide an ID when attempting to retrieve or update an object.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_RUN_AUDIT_BULK_WITH_SORTS = (
+        400,
+        "ATLAN-PYTHON-400-066",
+        "Unable to execute audit bulk search with user-defined sorting options.",
+        "Please ensure that no sorting options are included in your "
+        + "audit search request when performing a bulk search.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_RUN_SEARCH_LOG_BULK_WITH_SORTS = (
+        400,
+        "ATLAN-PYTHON-400-067",
+        "Unable to execute search log bulk search with user-defined sorting options.",
+        "Please ensure that no sorting options are included in your "
+        + "search log search request when performing a bulk search.",
+        InvalidRequestError,
+    )
+    MISSING_NAME = (
+        400,
+        "ATLAN-PYTHON-400-067",
+        "No name instance was provided when attempting to retrieve an object.",
+        "You must provide the name of the object when attempting to retrieve one.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_RETRIEVE_CLIENT_SECRET = (
+        400,
+        "ATLAN-PYTHON-400-068",
+        "Unable to fetch the client secret for GUID: {0}",
+        "Ensure the client GUID provided is correct and valid.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_RETRIEVE_USER_GUID = (
+        400,
+        "ATLAN-PYTHON-400-069",
+        "Unable to fetch the GUID for the user: {0}",
+        "Ensure the provided username is correct and valid.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_GENERATE_QN = (
+        400,
+        "ATLAN-PYTHON-400-070",
+        "Unable to generate qualifiedName for {0}, Error: {1}",
+        "Check the details of the server's message to correct your request.",
+        InvalidRequestError,
+    )
+    UNABLE_TO_CREATE_CREDENTIAL = (
+        400,
+        "ATLAN-PYTHON-400-071",
+        "Credentials cannot include a `username` or `password` when `test` is `False`.",
+        "Set `test` to `True` or remove `username` and `password`.",
+        InvalidRequestError,
+    )
+    MISSING_DATA_PRODUCT_ASSET_DSL = (
+        400,
+        "ATLAN-PYTHON-400-072",
+        "Missing value for `data_product_assets_d_s_l`, which is required to retrieve DataProduct assets.",
+        "Ensure your product instance has a valid `data_product_assets_d_s_l` value before making the request.",
         InvalidRequestError,
     )
     AUTHENTICATION_PASSTHROUGH = (
         401,
         "ATLAN-PYTHON-401-000",
-        "Server responded with {0}: {1}",
-        "Check the details of the server's message to correct your request.",
+        "Server responded with an authentication error {0}: {1} -- caused by: {2}",
+        "Your API or bearer token is either invalid or has expired, or you are "
+        + "attempting to access a URL you are not authorized to access. "
+        + "Ensure you are using a valid token, or try obtaining a new token and try again.",
         AuthenticationError,
     )
     NO_API_TOKEN = (
@@ -517,14 +696,28 @@ class ErrorCode(Enum):
     PERMISSION_PASSTHROUGH = (
         403,
         "ATLAN-PYTHON-403-000",
-        "Server responded with {0}: {1}",
+        "Server responded with a permission error {0}: {1} -- caused by: {2}",
         "Check the details of the server's message to correct your request.",
         PermissionError,
     )
+    UNABLE_TO_ESCALATE = (
+        403,
+        "ATLAN-PYTHON-403-001",
+        "Unable to escalate to a privileged user.",
+        "Check the details of your configured privileged credentials.",
+        PermissionError,
+    )
+    UNABLE_TO_IMPERSONATE = (
+        403,
+        "ATLAN-PYTHON-403-002",
+        "Unable to impersonate requested user.",
+        "Check the details of your configured privileged credentials and the user you requested to impersonate.",
+        PermissionError,
+    )
     NOT_FOUND_PASSTHROUGH = (
         404,
         "ATLAN-PYTHON-404-000",
-        "Server responded with {0}: {1}",
+        "Server responded with a not found error {0}: {1}.",
         "Check the details of the server's message to correct your request.",
         NotFoundError,
     )
@@ -725,10 +918,17 @@ class ErrorCode(Enum):
         "Verify the requested typedef exists in your Atlan environment.",
         NotFoundError,
     )
+    API_TOKEN_NOT_FOUND_BY_NAME = (
+        404,
+        "ATLAN-PYTHON-404-028",
+        "API token with name {0} does not exist.",
+        "Verify the API token provided is a valid username for that token.",
+        NotFoundError,
+    )
     CONFLICT_PASSTHROUGH = (
         409,
         "ATLAN-PYTHON-409-000",
-        "Server responded with {0}: {1}",
+        "Server responded with a conflict {0}: {1} -- caused by: {2}",
         "Check the details of the server's message to correct your request.",
         ConflictError,
     )
@@ -744,14 +944,14 @@ class ErrorCode(Enum):
     RATE_LIMIT_PASSTHROUGH = (
         429,
         "ATLAN-PYTHON-429-000",
-        "Server responded with {0}: {1}",
+        "Server responded with a rate limit violation {0}: {1} -- caused by: {2}",
         "Check the details of the server's message to correct your request.",
         RateLimitError,
     )
     ERROR_PASSTHROUGH = (
         500,
         "ATLAN-PYTHON-500-000",
-        "Server responded with {0}: {1}",
+        "Server responded with an error {0}: {1} -- caused by: {2}",
         "Check the details of the server's message to correct your request.",
         ApiError,
     )
@@ -765,7 +965,7 @@ class ErrorCode(Enum):
     UNABLE_TO_DESERIALIZE = (
         500,
         "ATLAN-PYTHON-500-002",
-        "Unable to deserialize value: [0]",
+        "Unable to deserialize value: {0}.",
         RAISE_GITHUB_ISSUE,
         LogicError,
     )
@@ -806,11 +1006,11 @@ class ErrorCode(Enum):
         self.user_action = user_action
         self.exception_type = exception_type
 
-    def exception_with_parameters(self, *args):
-        return self.exception_type(self, *args)
+    def exception_with_parameters(self, *args, **kwargs):
+        return self.exception_type(self, *args, **kwargs)
 
 
-ERROR_CODE_FOR_HTTP_STATUS: dict[int, ErrorCode] = {
+ERROR_CODE_FOR_HTTP_STATUS: Dict[int, ErrorCode] = {
     400: ErrorCode.INVALID_REQUEST_PASSTHROUGH,
     401: ErrorCode.AUTHENTICATION_PASSTHROUGH,
     403: ErrorCode.PERMISSION_PASSTHROUGH,
