@@ -1,10 +1,11 @@
 # SPDX-License-Identifier: Apache-2.0
-# Copyright 2022 Atlan Pte. Ltd.
+# Copyright 2025 Atlan Pte. Ltd.
 """
 This script can be used to generate the source code for pyatlan.model.assets, pyatlan.model.structs.py and part of
 pyatlan.model.enums. This script depends upon the presence of a JSON file containing typedefs downloaded from
 an Atlan instance. The script create_typedefs_file.py can be used to produce this file.
 """
+
 import datetime
 import enum
 import json
@@ -12,33 +13,33 @@ import os
 import re
 from enum import Enum
 from pathlib import Path
-from typing import Any, NamedTuple, Optional
+from typing import Any, Dict, List, NamedTuple, Optional, Set
 
 import networkx as nx
 from jinja2 import Environment, PackageLoader
 
-from pyatlan.model.core import to_snake_case
-from pyatlan.model.typedef import EntityDef, EnumDef, TypeDefResponse
+from pyatlan.model.typedef import EntityDef, EnumDef, RelationshipDef, TypeDefResponse
+from pyatlan.model.utils import to_python_class_name, to_snake_case
 
 REFERENCEABLE = "Referenceable"
 TYPE_DEF_FILE = Path(os.getenv("TMPDIR", "/tmp")) / "typedefs.json"
 TYPE_REPLACEMENTS = [
-    ("array<string>", "set[string]"),
-    ("array<date>", "set[date]"),
-    ("array<boolean>", "set[bool]"),
-    ("array<int>", "set[int]"),
-    ("array<float>", "set[float]"),
-    ("array<long>", "set[int]"),
+    ("array<string>", "Set[string]"),
+    ("array<date>", "Set[date]"),
+    ("array<boolean>", "Set[bool]"),
+    ("array<int>", "Set[int]"),
+    ("array<float>", "Set[float]"),
+    ("array<long>", "Set[int]"),
     ("icon_type", "IconType"),
     ("string", "str"),
     ("date", "datetime"),
-    ("array", "list"),
+    ("array", "List"),
     ("boolean", "bool"),
     ("float", "float"),
     ("long", "int"),
     ("__internal", "Internal"),
     ("certificate_status", "CertificateStatus"),
-    ("map", "dict"),
+    ("map", "Dict"),
     (">", "]"),
     ("<", "["),
     ("query_username_strategy", "QueryUsernameStrategy"),
@@ -53,6 +54,8 @@ TYPE_REPLACEMENTS = [
     ("file_type", "FileType"),
     ("atlas_operation", "AtlasOperation"),
     ("matillion_job_type", "MatillionJobType"),
+    ("incident_severity", "IncidentSeverity"),
+    ("table_type", "TableType"),
 ]
 PRIMITIVE_MAPPINGS = {
     "string": "str",
@@ -64,18 +67,25 @@ PRIMITIVE_MAPPINGS = {
     "string,string": "str, str",
     "string,long": "str, int",
 }
-ARRAY_REPLACEMENTS = [("array<string>", "set{string}")]
+ARRAY_REPLACEMENTS = [("array<string>", "Set{string}")]
 ADDITIONAL_IMPORTS = {
     "datetime": "from datetime import datetime",
     "CertificateStatus": "from .enums import CertificateStatus",
     "SourceCostUnitType": "from .enums import SourceCostUnitType",
     "PopularityInsights": "from .structs import PopularityInsights",
 }
-PARENT = Path(__file__).parent
+# The resolve() method ensures
+# the retrieval of the absolute path for the file,
+# helping to prevent inconsistencies
+# across Python versions (e.g: 3.8 and 3.9).
+PARENT = Path(__file__).resolve().parent
 ASSETS_DIR = PARENT.parent / "model" / "assets"
+ASSETS_RELATIONS_DIR = PARENT.parent / "model" / "assets" / "relations"
+CORE_ASSETS_DIR = PARENT.parent / "model" / "assets" / "core"
 MODEL_DIR = PARENT.parent / "model"
 DOCS_DIR = PARENT.parent / "documentation"
 SPHINX_DIR = PARENT.parent.parent / "docs"
+TEMPLATES_DIR = PARENT / "templates"
 
 
 def get_type(type_: str):
@@ -85,17 +95,23 @@ def get_type(type_: str):
     return ret_value
 
 
-def get_type_defs() -> TypeDefResponse:
+def get_type_defs(typedefs_file_path=None) -> TypeDefResponse:
+    # Use provided path or default to tmp directory
+    if typedefs_file_path:
+        typedef_file = Path(typedefs_file_path)
+    else:
+        typedef_file = TYPE_DEF_FILE
+
     if (
-        not TYPE_DEF_FILE.exists()
-        or datetime.date.fromtimestamp(os.path.getmtime(TYPE_DEF_FILE))
+        not typedef_file.exists()
+        or datetime.date.fromtimestamp(os.path.getmtime(typedef_file))
         < datetime.date.today()
     ):
         raise ClassGenerationError(
             "File containing typedefs does not exist or is not current."
-            f" Please run create_typedefs_file to create {TYPE_DEF_FILE}."
+            f" Please run create_typedefs_file to create {typedef_file}."
         )
-    with TYPE_DEF_FILE.open() as input_file:
+    with typedef_file.open() as input_file:
         return TypeDefResponse(**json.load(input_file))
 
 
@@ -110,8 +126,9 @@ class ModuleStatus(str, Enum):
 
 class ModuleInfo:
     count: int = 0
-    modules: set["ModuleInfo"] = set()
-    modules_by_asset_name: dict[str, str] = {}
+    modules: Set["ModuleInfo"] = set()
+    modules_by_asset_name: Dict[str, str] = {}
+    assets: Dict[str, "AssetInfo"] = {}
 
     @classmethod
     def check_for_circular_module_dependencies(cls):
@@ -131,7 +148,7 @@ class ModuleInfo:
     def __init__(self, asset_info: "AssetInfo"):
         self.order = ModuleInfo.count
         self._name = f"asset{ModuleInfo.count:02d}"
-        self.asset_infos: set["AssetInfo"] = set()
+        self.asset_infos: Set["AssetInfo"] = set()
         self.add_asset_info(asset_info=asset_info)
         self.status: ModuleStatus = ModuleStatus.ACTIVE
         ModuleInfo.modules.add(self)
@@ -197,19 +214,48 @@ class ModuleInfo:
 
 
 class AssetInfo:
-    asset_info_by_name: dict[str, "AssetInfo"] = {}
+    asset_info_by_name: Dict[str, "AssetInfo"] = {}
     hierarchy_graph: nx.DiGraph = nx.DiGraph()
-    super_type_names_to_ignore: set[str] = set()
-    entity_defs_by_name: dict[str, EntityDef] = {}
+    super_type_names_to_ignore: Set[str] = set()
+    entity_defs_by_name: Dict[str, EntityDef] = {}
+    sub_type_names_to_ignore: Set[str] = set()
+    is_core_asset: bool = False
+    _ASSETS_REQUIRE_CLIENT = {
+        "Badge",
+        "Collection",
+        "Connection",
+        "DataProduct",
+        "Referenceable",
+        "Purpose",
+    }
+    _CORE_ASSETS = {
+        "Referenceable",
+        "Asset",
+        "AuthPolicy",
+        "PowerBIDatasource",
+        "PowerBIReport",
+        "PowerBITile",
+        "PowerBIColumn",
+        "PowerBIMeasure",
+        "MatillionGroup",
+        "Stakeholder",
+        "StakeholderTitle",
+        "NoSQL",
+        "DocumentDBCollection",
+        "FlowDataset",
+        "DatabricksAIModelVersion",
+    }
 
     def __init__(self, name: str, entity_def: EntityDef):
         self._name = name
         self.entity_def: EntityDef = entity_def
         self.update_attribute_defs()
         self.module_info: Optional[ModuleInfo] = None
-        self.required_asset_infos: set["AssetInfo"] = set()
-        self.circular_dependencies: set["AssetInfo"] = set()
+        self.required_asset_infos: Set["AssetInfo"] = set()
+        self.circular_dependencies: Set["AssetInfo"] = set()
         self.order: int = 0
+        self.module_name = to_snake_case(name)
+        self.super_type: Optional[AssetInfo] = None
 
     def __hash__(self):
         return hash(self._name)
@@ -228,6 +274,36 @@ class AssetInfo:
         else:
             return self.entity_def.super_types[0]
 
+    @property
+    def import_super_class(self):
+        if self._name == REFERENCEABLE:
+            return ""
+        super_type = AssetInfo.asset_info_by_name[self.entity_def.super_types[0]]
+        if self.name not in self._CORE_ASSETS and super_type.name in self._CORE_ASSETS:
+            return f"from .core.{super_type.module_name} import {super_type.name}"
+        elif (
+            not self.is_core_asset
+            and super_type.is_core_asset
+            and self.name not in self._CORE_ASSETS
+        ):
+            return f"from .core.{super_type.module_name} import {super_type.name}"
+        else:
+            return f"from .{super_type.module_name} import {super_type.name}"
+
+    @property
+    def imports_for_referenced_assets(self):
+        imports = []
+
+        for required_asset in self.required_asset_infos:
+            if not self.is_core_asset and required_asset.is_core_asset:
+                import_statement = f"from .core.{required_asset.module_name} import {required_asset.name} # noqa: E402, F401"
+            else:
+                import_statement = f"from .{required_asset.module_name} import {required_asset.name} # noqa: E402, F401"
+
+            imports.append(import_statement)
+
+        return imports
+
     def update_attribute_defs(self):
         def get_ancestor_relationship_defs(
             ancestor_name: str, ancestor_relationship_defs
@@ -240,9 +316,11 @@ class AssetInfo:
             ):
                 ancestor_relationship_defs.add(relationship_def["name"])
             return get_ancestor_relationship_defs(
-                ancestor_entity_def.super_types[0]
-                if ancestor_entity_def.super_types
-                else "",
+                (
+                    ancestor_entity_def.super_types[0]
+                    if ancestor_entity_def.super_types
+                    else ""
+                ),
                 ancestor_relationship_defs,
             )
 
@@ -265,7 +343,7 @@ class AssetInfo:
         )
 
     def update_required_asset_names(self) -> None:
-        attributes_to_remove: set[str] = set()
+        attributes_to_remove: Set[str] = set()
         attribute_defs = self.entity_def.attribute_defs or []
         relationship_attribute_defs = self.entity_def.relationship_attribute_defs or []
         for attribute in attribute_defs + relationship_attribute_defs:
@@ -284,10 +362,6 @@ class AssetInfo:
             for a in relationship_attribute_defs
             if a["name"] not in attributes_to_remove
         ]
-        if self.entity_def.super_types:
-            self.required_asset_infos.add(
-                AssetInfo.asset_info_by_name[self.entity_def.super_types[0]]
-            )
 
     def merge_attributes(self, entity_def):
         def merge_them(s, a):
@@ -317,7 +391,7 @@ class AssetInfo:
                 self.circular_dependencies.add(super_type)
 
     @classmethod
-    def set_entity_defs(cls, entity_defs: list[EntityDef]):
+    def set_entity_defs(cls, entity_defs: List[EntityDef]):
         cls.entity_defs_by_name = {
             entity_def.name: entity_def for entity_def in entity_defs
         }
@@ -336,7 +410,8 @@ class AssetInfo:
                 cls.super_type_names_to_ignore.add(name)
                 continue
             for asset_name in entity_def.sub_types or []:
-                AssetInfo.hierarchy_graph.add_edge(name, asset_name)
+                if asset_name not in AssetInfo.sub_type_names_to_ignore:
+                    AssetInfo.hierarchy_graph.add_edge(name, asset_name)
             asset_info = AssetInfo(name=name, entity_def=entity_def)
             AssetInfo.asset_info_by_name[name] = asset_info
         for asset_info in AssetInfo.asset_info_by_name.values():
@@ -357,10 +432,50 @@ class AssetInfo:
                 asset_info = cls.asset_info_by_name[asset_name]
                 asset_info.order = order
                 order += 1
-                if asset_info.module_info is None:
-                    ModuleInfo(asset_info=asset_info)
-                else:
-                    asset_info.module_info.add_asset_info(asset_info=asset_info)
+
+                ModuleInfo.assets[asset_name] = asset_info
+                if asset_info.name in cls._CORE_ASSETS:
+                    if asset_info.super_class != "AtlanObject":
+                        super_asset = cls.asset_info_by_name[asset_info.super_class]
+                        super_asset.is_core_asset = True
+                        cls._CORE_ASSETS.add(asset_info.super_class)
+                    for related_asset in asset_info.required_asset_infos:
+                        if related_asset.is_core_asset:
+                            continue
+                        related_asset.is_core_asset = True
+                        cls._CORE_ASSETS.add(related_asset.name)
+
+                if asset_info.super_class in cls._CORE_ASSETS:
+                    if asset_info.required_asset_infos:
+                        for related_asset in asset_info.required_asset_infos:
+                            if related_asset.is_core_asset:
+                                asset_info.is_core_asset = True
+                                cls._CORE_ASSETS.add(asset_info.name)
+                                continue
+                            super_asset = cls.asset_info_by_name[
+                                related_asset.super_class
+                            ]
+                            super_asset.is_core_asset = True
+                            cls._CORE_ASSETS.add(related_asset.super_class)
+
+
+class RelationshipDefInfo:
+    module_name: str
+    relationship_def_infos: List["RelationshipDefInfo"] = []
+
+    def __init__(self, name: str, relationship_def: RelationshipDef):
+        self.module_name = to_snake_case(name)
+        self.relationship_def = relationship_def
+
+    @classmethod
+    def create(cls, relationship_defs: List[RelationshipDef]):
+        for rel_def in relationship_defs:
+            if rel_def.attribute_defs:
+                cls.relationship_def_infos.append(
+                    RelationshipDefInfo(
+                        name=to_snake_case(rel_def.name), relationship_def=rel_def
+                    )
+                )
 
 
 class AttributeType(Enum):
@@ -410,13 +525,13 @@ def get_mapped_type(type_name: str) -> MappedType:
         if type_name.startswith("array<"):
             if type_name.startswith("array<map<"):
                 base_type = get_embedded_type(type_name[len("array<") :])  # noqa: E203
-                container = "list[dict["
+                container = "List[Dict["
             else:
                 base_type = get_embedded_type(type_name)
-                container = "set["
+                container = "Set["
         elif type_name.startswith("map<"):
             base_type = get_embedded_type(type_name)
-            container = "dict["
+            container = "Dict["
     builder = MappedType()
     builder.original_base = base_type
     if primitive_name := PRIMITIVE_MAPPINGS.get(base_type):
@@ -429,7 +544,7 @@ def get_mapped_type(type_name: str) -> MappedType:
             builder.name = mapped_type.name
             if base_type_of_mapped == AttributeType.STRUCT:
                 # If the referred object is a struct, change the container to a list rather than a set
-                container = "list["
+                container = "List["
         else:
             # Failing any cached type, fallback to just the name of the object
             builder.attr_type = AttributeType.ASSET
@@ -440,6 +555,7 @@ def get_mapped_type(type_name: str) -> MappedType:
 
 
 def get_class_var_for_attr(attr_name: str) -> str:
+    attr_name = re.sub(r"GUIDs$", "Guids", attr_name)
     replace1 = re.sub(r"([A-Z]+)([A-Z][a-z])", r"\1_\2", attr_name.replace("_", ""))
     replace2 = re.sub(r"([a-z])([A-Z])", r"\1_\2", replace1)
     return replace2.upper()
@@ -464,12 +580,14 @@ class SearchType:
         self.args = args
 
 
-def get_search_type(attr_def: dict[str, Any]) -> SearchType:
+def get_search_type(attr_def: Dict[str, Any]) -> SearchType:
     def get_default_index_for_type(base_type: str) -> IndexType:
         if base_type in {"date", "float", "double", "int", "long"}:
             to_use = IndexType.NUMERIC
         elif base_type == "boolean":
             to_use = IndexType.BOOLEAN
+        elif base_type == "string":
+            to_use = IndexType.TEXT
         else:
             to_use = IndexType.KEYWORD
         return to_use
@@ -492,8 +610,8 @@ def get_search_type(attr_def: dict[str, Any]) -> SearchType:
                 base_type = get_embedded_type(type_name)
         return base_type
 
-    def get_indexes_for_attribute() -> dict[IndexType, str]:
-        searchable: dict[IndexType, str] = {}
+    def get_indexes_for_attribute() -> Dict[IndexType, str]:
+        searchable: Dict[IndexType, str] = {}
         config = attr_def.get("indexTypeESConfig")
         attr_name = str(attr_def.get("name"))
         if "relationshipTypeName" in attr_def:
@@ -507,6 +625,8 @@ def get_search_type(attr_def: dict[str, Any]) -> SearchType:
                         searchable[IndexType.STEMMED] = attr_name
                     else:
                         searchable[IndexType.TEXT] = attr_name
+            elif attr_def.get("indexType") == "STRING":
+                searchable[IndexType.KEYWORD] = attr_name
             else:
                 def_index = get_default_index_for_type(base_type)
                 searchable[def_index] = attr_name
@@ -548,14 +668,12 @@ def get_search_type(attr_def: dict[str, Any]) -> SearchType:
     elif indices == {IndexType.NUMERIC, IndexType.RANK_FEATURE}:
         return SearchType(
             name="NumericRankField",
-            args=f'"{search_map.get(IndexType.NUMERIC)}", '
-            f'"{search_map.get(IndexType.RANK_FEATURE)}"',
+            args=f'"{search_map.get(IndexType.NUMERIC)}", "{search_map.get(IndexType.RANK_FEATURE)}"',
         )
     elif indices == {IndexType.KEYWORD, IndexType.TEXT}:
         return SearchType(
             name="KeywordTextField",
-            args=f'"{search_map.get(IndexType.KEYWORD)}", '
-            f'"{search_map.get(IndexType.TEXT)}"',
+            args=f'"{search_map.get(IndexType.KEYWORD)}", "{search_map.get(IndexType.TEXT)}"',
         )
     elif indices == {IndexType.KEYWORD, IndexType.TEXT, IndexType.STEMMED}:
         return SearchType(
@@ -574,6 +692,7 @@ class Generator:
         )
         self.environment.filters["to_snake_case"] = to_snake_case
         self.environment.filters["get_type"] = get_type
+        self.environment.filters["to_cls_name"] = to_python_class_name
         self.environment.filters["get_search_type"] = get_search_type
         self.environment.filters["get_mapped_type"] = get_mapped_type
         self.environment.filters["get_class_var_for_attr"] = get_class_var_for_attr
@@ -605,47 +724,109 @@ class Generator:
         for relationship_def in ancestor_entity_def.relationship_attribute_defs or []:
             ancestor_relationship_defs.add(relationship_def["name"])
         return self.get_ancestor_relationship_defs(
-            ancestor_entity_def.super_types[0]
-            if ancestor_entity_def.super_types
-            else "",
+            (
+                ancestor_entity_def.super_types[0]
+                if ancestor_entity_def.super_types
+                else ""
+            ),
             ancestor_relationship_defs,
         )
 
-    def render_modules(self, modules: list[ModuleInfo]):
-        for module in modules:
-            self.render_module(module)
-        self.render_init(modules)
+    def render_modules(self, modules: List[ModuleInfo]):
+        self.render_init(modules)  # type: ignore
 
-    def render_module(self, module: ModuleInfo):
+    def render_module(self, asset_info: AssetInfo, enum_defs: List["EnumDefInfo"]):
         template = self.environment.get_template("module.jinja2")
         content = template.render(
             {
-                "module": module,
+                "asset_info": asset_info,
                 "existz": os.path.exists,
-                "module_name": module.name,
-                "modules_by_asset_name": ModuleInfo.modules_by_asset_name,
+                "enum_defs": enum_defs,
+                "templates_path": TEMPLATES_DIR.absolute().as_posix(),
             }
         )
-        with (ASSETS_DIR / f"{module.name}.py").open("w") as script:
+        with (ASSETS_DIR / f"{asset_info.module_name}.py").open("w") as script:
             script.write(content)
 
-    def render_init(self, modules: list[ModuleInfo]):
-        imports = sorted(
-            {
-                f"from .{asset_info.module_info.name} import {asset_info.name}"
-                for module in modules
-                if module.status == ModuleStatus.ACTIVE
-                for asset_info in module.asset_infos
-                if asset_info.module_info
-            }
-        )
-        template = self.environment.get_template("init.jinja2")
+    def render_custom_relationship_module(
+        self, relationship_def_info: RelationshipDefInfo
+    ):
+        template = self.environment.get_template("custom_relationship.jinja2")
         content = template.render(
             {
-                "imports": imports,
+                "relationship_info": relationship_info.relationship_def,
+                "templates_path": TEMPLATES_DIR.absolute().as_posix(),
             }
         )
-        with (ASSETS_DIR / "__init__.py").open("w") as script:
+        with (ASSETS_RELATIONS_DIR / f"{relationship_def_info.module_name}.py").open(
+            "w"
+        ) as script:
+            script.write(content)
+
+    def render_core_module(self, asset_info: AssetInfo, enum_defs: List["EnumDefInfo"]):
+        template = self.environment.get_template("module.jinja2")
+        content = template.render(
+            {
+                "asset_info": asset_info,
+                "existz": os.path.exists,
+                "enum_defs": enum_defs,
+                "templates_path": TEMPLATES_DIR.absolute().as_posix(),
+            }
+        )
+        with (CORE_ASSETS_DIR / f"{asset_info.module_name}.py").open("w") as script:
+            script.write(content)
+
+    def render_init(self, assets: List[AssetInfo]):
+        template = self.environment.get_template("init.jinja2")
+        content = template.render({"assets": assets})
+
+        init_path = ASSETS_DIR / "__init__.py"
+        with init_path.open("w") as script:
+            script.write(content)
+
+    def render_core_init(self, assets: List[AssetInfo]):
+        asset_names = [
+            asset.name
+            for asset in assets
+            if asset.is_core_asset or asset.name in asset._CORE_ASSETS
+        ]
+        asset_imports = [
+            f"from .{asset.module_name} import {asset.name}"
+            for asset in assets
+            if asset.is_core_asset or asset.name in asset._CORE_ASSETS
+        ]
+
+        template = self.environment.get_template("core/init.jinja2")
+        content = template.render(
+            {"asset_imports": asset_imports, "asset_names": asset_names}
+        )
+
+        init_path = CORE_ASSETS_DIR / "__init__.py"
+        with init_path.open("w") as script:
+            script.write(content)
+
+    def render_mypy_init(self, assets: List[AssetInfo]):
+        template = self.environment.get_template("mypy_init.jinja2")
+        content = template.render({"assets": assets})
+
+        init_path = ASSETS_DIR / "__init__.pyi"
+        with init_path.open("w") as script:
+            script.write(content)
+
+    def render_relations_init(self, relation_def_infos: List[RelationshipDefInfo]):
+        template = self.environment.get_template("relations_init.jinja2")
+        content = template.render({"relation_def_infos": relation_def_infos})
+
+        init_path = ASSETS_RELATIONS_DIR / "__init__.py"
+        with init_path.open("w") as script:
+            script.write(content)
+
+    def render_relations_mypy_init(self, relation_def_infos: List[RelationshipDefInfo]):
+        template = self.environment.get_template("relations_mypy_init.jinja2")
+        content = template.render({"relation_def_infos": relation_def_infos})
+
+        init_path = ASSETS_RELATIONS_DIR / "__init__.pyi"
+        with init_path.open("w") as script:
             script.write(content)
 
     def render_structs(self, struct_defs):
@@ -654,7 +835,7 @@ class Generator:
         with (MODEL_DIR / "structs.py").open("w") as script:
             script.write(content)
 
-    def render_enums(self, enum_defs: list["EnumDefInfo"]):
+    def render_enums(self, enum_defs: List["EnumDefInfo"]):
         template = self.environment.get_template("enums.jinja2")
         content = template.render({"enum_defs": enum_defs})
         start_of_generated_section_found = False
@@ -758,11 +939,11 @@ class KeyValue(NamedTuple):
 
 
 class EnumDefInfo:
-    enum_def_info: list["EnumDefInfo"] = []
+    enum_def_info: List["EnumDefInfo"] = []
 
     def __init__(self, enum_def: EnumDef):
         self.name = get_type(enum_def.name)
-        self.element_defs: list[KeyValue] = [
+        self.element_defs: List[KeyValue] = [
             self.get_key_value(e)
             for e in sorted(enum_def.element_defs, key=lambda e: e.ordinal or 0)
         ]
@@ -779,26 +960,88 @@ class EnumDefInfo:
     @classmethod
     def create(cls, enum_defs):
         for enum_def in enum_defs:
-            cls.enum_def_info.append(EnumDefInfo(enum_def))
+            # Only pick `atlas_core` enums, not user-created ones.
+            if enum_def.service_type == "atlas_core":
+                cls.enum_def_info.append(EnumDefInfo(enum_def))
         cls.enum_def_info = sorted(cls.enum_def_info, key=lambda e: e.name)
 
 
+def filter_attributes_of_custom_entity_type():
+    for entity_def in type_defs.reserved_entity_defs:
+        if entity_def.attribute_defs:
+            filtered_attribute_defs = [
+                attribute_def
+                for attribute_def in entity_def.attribute_defs
+                if not type_defs.is_custom_entity_def_name(attribute_def["typeName"])
+            ]
+            entity_def.attribute_defs = filtered_attribute_defs
+        if entity_def.relationship_attribute_defs:
+            filtered_relationship_attribute_defs = [
+                relationship_attribute_def
+                for relationship_attribute_def in entity_def.relationship_attribute_defs
+                if not type_defs.is_custom_entity_def_name(
+                    relationship_attribute_def["typeName"]
+                )
+            ]
+            entity_def.relationship_attribute_defs = (
+                filtered_relationship_attribute_defs
+            )
+
+
 if __name__ == "__main__":
-    type_defs = get_type_defs()
-    AssetInfo.set_entity_defs(type_defs.entity_defs)
+    import argparse
+
+    parser = argparse.ArgumentParser(
+        description="Generate Atlan SDK classes from typedefs"
+    )
+    parser.add_argument(
+        "--typedefs-file",
+        type=str,
+        help="Path to the typedefs file (default: /tmp/typedefs.json or $TMPDIR/typedefs.json)",
+    )
+    args = parser.parse_args()
+
+    type_defs = get_type_defs(args.typedefs_file)
+    filter_attributes_of_custom_entity_type()
+    AssetInfo.sub_type_names_to_ignore = type_defs.custom_entity_def_names
+    AssetInfo.set_entity_defs(type_defs.reserved_entity_defs)
+    RelationshipDefInfo.create(type_defs.relationship_defs)
     AssetInfo.update_all_circular_dependencies()
     AssetInfo.create_modules()
-    ModuleInfo.check_for_circular_module_dependencies()
+
     for file in (ASSETS_DIR).glob("*.py"):
         file.unlink()
+    for file in (CORE_ASSETS_DIR).glob("*.py"):
+        # Remove all files in the core assets directory except `indistinct.py`
+        if file.name == "indistinct_asset.py":
+            continue
+        file.unlink()
     generator = Generator()
-    generator.render_modules(
-        [m for m in ModuleInfo.modules if m.status == ModuleStatus.ACTIVE]
-    )
-    generator.render_structs(type_defs.struct_defs)
     EnumDefInfo.create(type_defs.enum_defs)
+
+    for asset_info in ModuleInfo.assets.values():
+        if asset_info.is_core_asset or asset_info.name in asset_info._CORE_ASSETS:
+            generator.render_core_module(asset_info, EnumDefInfo.enum_def_info)
+        else:
+            generator.render_module(asset_info, EnumDefInfo.enum_def_info)
+
+    for file in (ASSETS_RELATIONS_DIR).glob("*.py"):
+        # Ignore non-generator files
+        if file.name in ("relationship_attributes.py", "indistinct_relationship.py"):
+            continue
+        file.unlink()
+
+    for relationship_info in RelationshipDefInfo.relationship_def_infos:
+        generator.render_custom_relationship_module(relationship_info)
+
+    generator.render_init(ModuleInfo.assets.values())  # type: ignore
+    generator.render_core_init(ModuleInfo.assets.values())  # type: ignore
+    generator.render_mypy_init(ModuleInfo.assets.values())  # type: ignore
+    generator.render_relations_init(RelationshipDefInfo.relationship_def_infos)  # type: ignore
+    generator.render_relations_mypy_init(RelationshipDefInfo.relationship_def_infos)  # type: ignore
+    generator.render_structs(type_defs.struct_defs)
     generator.render_enums(EnumDefInfo.enum_def_info)
     generator.render_docs_struct_snippets(type_defs.struct_defs)
-    generator.render_docs_entity_properties(type_defs.entity_defs)
-    generator.render_docs_entity_relationships(type_defs.entity_defs)
-    generator.render_sphinx_docs(type_defs.entity_defs)
+    generator.render_docs_entity_properties(type_defs.reserved_entity_defs)
+    generator.render_docs_entity_relationships(type_defs.reserved_entity_defs)
+    generator.render_sphinx_docs(type_defs.reserved_entity_defs)
