@@ -3,9 +3,9 @@
 from abc import ABC
 from datetime import date
 from enum import Enum
-from typing import Union, overload
+from typing import Any, Dict, List, Optional, Union, overload
 
-from pydantic import StrictBool, StrictFloat, StrictInt, StrictStr
+from pydantic.v1 import StrictBool, StrictFloat, StrictInt, StrictStr
 
 from pyatlan.errors import ErrorCode
 from pyatlan.model.aggregation import Aggregation
@@ -14,13 +14,16 @@ from pyatlan.model.enums import AtlanComparisonOperator, SortOrder
 from pyatlan.model.search import (
     Exists,
     Match,
+    MatchPhrase,
     Prefix,
     Query,
     Range,
+    Regexp,
     SearchFieldType,
     SortItem,
     Term,
     Terms,
+    Wildcard,
 )
 from pyatlan.model.typedef import AttributeDef
 from pyatlan.utils import ComparisonCategory, is_comparable_type
@@ -56,6 +59,7 @@ class SearchableField(AtlanField):
     """
 
     elastic_field_name: StrictStr
+    EMBEDDED_SOURCE_VALUE = "sourceValue"
 
     def __init__(self, atlan_field_name: StrictStr, elastic_field_name: StrictStr):
         """
@@ -92,15 +96,49 @@ class SearchableField(AtlanField):
     def bucket_by(
         self,
         size: int = 10,
+        include_source_value: bool = False,
+        nested: Optional[Dict[Any, Aggregation]] = None,
+        order: Optional[List[Dict[str, SortOrder]]] = None,
     ) -> Aggregation:
-        """Return criteria to bucket results based on the provided field.
-           :param size: the number of buckets to include results across.
-           :returns: criteria to bucket results by the provided field, across a maximum number of buckets defined by
-           the provided size
-        */"""
-        return Aggregation(
-            __root__={"terms": {"field": self.elastic_field_name, "size": size}}
-        )
+        """
+        Return criteria to bucket results based on the provided field.
+
+        :param size: the number of buckets to include results across, defaults to `10`.
+        :param include_source_value: whether to include the source value (`True`) or not (`False`)
+        :param nested: (optional) nested aggregations to include.
+        :param order: (optional) the order for the buckets.
+        :returns: criteria to bucket results by the provided field,
+        across a maximum number of buckets defined by the provided size.
+        """
+        aggs = {"terms": {"field": self.elastic_field_name, "size": size}}
+
+        if include_source_value and not nested:
+            source_field = (
+                self.elastic_field_name
+                # Need to handle the hashed-string ID stuff for custom metadata fields
+                if isinstance(self, CustomMetadataField)
+                else self.atlan_field_name
+            )
+            nested = {
+                self.EMBEDDED_SOURCE_VALUE: Aggregation(
+                    __root__={
+                        "top_hits": {
+                            "size": 1,
+                            "_source": {"includes": [source_field]},
+                        }
+                    }
+                )
+            }
+
+        if nested:
+            aggs_nested = {
+                **aggs,
+                "aggregations": nested,
+                **({"order": order} if order else {}),
+            }
+            return Aggregation(__root__=aggs_nested)
+
+        return Aggregation(__root__=aggs)
 
 
 class BooleanField(SearchableField):
@@ -203,7 +241,7 @@ class KeywordField(SearchableField):
             case_insensitive=case_insensitive,
         )
 
-    def within(self, values: list[str]) -> Query:
+    def within(self, values: List[str]) -> Query:
         """
         Returns a query that will match all assets whose field has a value that exactly matches
         at least one of the provided string values.
@@ -214,6 +252,38 @@ class KeywordField(SearchableField):
         """
         return Terms(field=self.keyword_field_name, values=values)
 
+    def wildcard(self, value: StrictStr, case_insensitive: bool = False) -> Query:
+        """
+        Returns a query that retrieves assets whose attribute value matches the
+        provided wildcard pattern. This function is particularly useful for searching
+        based on simple naming conventions.
+
+        :param value: The wildcard pattern to match against the asset's attribute value.
+        :param case_insensitive: If `True`, performs a case-insensitive match. Defaults to `False`.
+        :return: A query that matches assets with the specified wildcard pattern for the designated attribute.
+        """
+        return Wildcard(
+            field=self.keyword_field_name,
+            value=value,
+            case_insensitive=case_insensitive,
+        )
+
+    def regexp(self, value: StrictStr, case_insensitive: bool = False) -> Query:
+        """
+        Returns a query that retrieves assets whose attribute value matches the
+        provided regular expression. This function is particularly useful for
+        searching based on more complicated naming conventions.
+
+        :param value: The regular expression to match against the asset's attribute value.
+        :param case_insensitive: If `True`, performs a case-insensitive match. Defaults to `False`.
+        :return: A query that matches assets with the specified regex pattern for the designated attribute.
+        """
+        return Regexp(
+            field=self.keyword_field_name,
+            value=value,
+            case_insensitive=case_insensitive,
+        )
+
 
 class TextField(SearchableField):
     """
@@ -258,6 +328,51 @@ class TextField(SearchableField):
             query=value,
         )
 
+    def match_phrase(self, value: StrictStr) -> Query:
+        """
+        Returns a query that will textually match the specified phrase within the field,
+        ensuring that the words appear in the exact order provided. This analyzes the
+        provided value according to the same analysis carried out on the field
+        (for example, tokenization, stemming, and so on).
+
+        :param value: the string value to match against
+        :returns: a query that will only match assets whose analyzed value for the field matches the value provided
+                  (which will also be analyzed)
+        """
+        return MatchPhrase(
+            field=self.text_field_name,
+            query=value,
+        )
+
+
+class InternalKeywordField(KeywordField):
+    """
+    Represents any field in Atlan that can be searched only by keyword (no text-analyzed fuzziness), and can also be
+    searched against a special internal field directly within Atlan.
+    """
+
+    _internal_field_name: StrictStr
+
+    def __init__(
+        self,
+        atlan_field_name: StrictStr,
+        keyword_field_name: StrictStr,
+        internal_field_name: StrictStr,
+    ):
+        """
+        Default constructor.
+
+        :param atlan_field_name: name of the attribute in the metastore
+        :param keyword_field_name: name of the keyword field in the search index
+        :param internal_field_name: internal name of the internal searchable attribute in the metastore
+        """
+        super().__init__(atlan_field_name, keyword_field_name)
+        self._internal_field_name = internal_field_name
+
+    @property
+    def internal_field_name(self):
+        return self._internal_field_name
+
 
 class NumericField(SearchableField):
     """
@@ -369,6 +484,28 @@ class NumericField(SearchableField):
         return Aggregation(__root__={"max": {"field": self.elastic_field_name}})
 
 
+class InternalNumericField(NumericField):
+    def __init__(
+        self,
+        atlan_field_name: StrictStr,
+        numeric_field_name: StrictStr,
+        internal_field_name: StrictStr,
+    ):
+        """
+        Default constructor.
+
+        :param atlan_field_name: name of the attribute in the metastore
+        :param numeric_field_name: name of the numeric field in the search index
+        :param internal_field_name: internal name of the internal searchable attribute in the metastore
+        """
+        super().__init__(atlan_field_name, numeric_field_name)
+        self._internal_field_name = internal_field_name
+
+    @property
+    def internal_field_name(self):
+        return self._internal_field_name
+
+
 class NumericRankField(NumericField):
     """
     Represents any field in Atlan that can be searched using only numeric search operations,
@@ -498,22 +635,22 @@ class CustomMetadataField(SearchableField):
     attribute_name: str
     attribute_def: AttributeDef
 
-    def __init__(self, set_name: str, attribute_name: str):
-        from pyatlan.cache.custom_metadata_cache import CustomMetadataCache
-
+    def __init__(self, client: Any, set_name: str, attribute_name: str):
         super().__init__(
             StrictStr(
-                CustomMetadataCache.get_attribute_for_search_results(
+                client.custom_metadata_cache.get_attribute_for_search_results(
                     set_name, attribute_name
                 )
             ),
             StrictStr(
-                CustomMetadataCache.get_attr_id_for_name(set_name, attribute_name)
+                client.custom_metadata_cache.get_attr_id_for_name(
+                    set_name, attribute_name
+                )
             ),
         )
         self.set_name = set_name
         self.attribute_name = attribute_name
-        self.attribute_def = CustomMetadataCache.get_attribute_def(
+        self.attribute_def = client.custom_metadata_cache.get_attribute_def(
             self.elastic_field_name
         )
 
@@ -549,7 +686,7 @@ class CustomMetadataField(SearchableField):
             case_insensitive=case_insensitive,
         )
 
-    def within(self, values: list[str]) -> Query:
+    def within(self, values: List[str]) -> Query:
         """
         Returns a query that will match all assets whose field has a value that exactly matches
         at least one of the provided string values.
@@ -783,7 +920,7 @@ class LineageFilterFieldCM(LineageFilterField):
                 self._cm_field.attribute_def.type_name or "", ComparisonCategory.BOOLEAN
             ):
                 raise ErrorCode.INVALID_QUERY.exception_with_parameters(
-                    AtlanComparisonOperator.EQ,
+                    AtlanComparisonOperator.EQ.value,
                     f"{self._cm_field.set_name}.{self._cm_field.attribute_name}",
                 )
             return LineageFilter(
@@ -857,7 +994,7 @@ class LineageFilterFieldCM(LineageFilterField):
                 self._cm_field.attribute_def.type_name or "", ComparisonCategory.BOOLEAN
             ):
                 raise ErrorCode.INVALID_QUERY.exception_with_parameters(
-                    AtlanComparisonOperator.NEQ,
+                    AtlanComparisonOperator.NEQ.value,
                     f"{self._cm_field.set_name}.{self._cm_field.attribute_name}",
                 )
             return LineageFilter(
@@ -1093,13 +1230,16 @@ class LineageFilterFieldCM(LineageFilterField):
         comparison_operator: AtlanComparisonOperator,
         expected_types: str = "int, float or date",
     ):
-        if isinstance(
-            value,
-            bool,  # needed because isinstance(value, int) evaluates to true when value is bool
-        ) or (
-            not isinstance(value, int)
-            and not isinstance(value, float)
-            and not isinstance(value, date)
+        if (
+            isinstance(
+                value,
+                bool,  # needed because isinstance(value, int) evaluates to true when value is bool
+            )
+            or (
+                not isinstance(value, int)
+                and not isinstance(value, float)
+                and not isinstance(value, date)
+            )
         ):
             raise ErrorCode.INVALID_PARAMETER_TYPE.exception_with_parameters(
                 type(value).__name__, expected_types
@@ -1108,7 +1248,7 @@ class LineageFilterFieldCM(LineageFilterField):
             self._cm_field.attribute_def.type_name or "", ComparisonCategory.NUMBER
         ):
             raise ErrorCode.INVALID_QUERY.exception_with_parameters(
-                comparison_operator,
+                comparison_operator.value,
                 f"{self._cm_field.set_name}.{self._cm_field.attribute_name}",
             )
         return LineageFilter(
@@ -1126,7 +1266,7 @@ class LineageFilterFieldCM(LineageFilterField):
             self._cm_field.attribute_def.type_name or "", ComparisonCategory.STRING
         ):
             raise ErrorCode.INVALID_QUERY.exception_with_parameters(
-                comparison_operator,
+                comparison_operator.value,
                 f"{self._cm_field.set_name}.{self._cm_field.attribute_name}",
             )
         return LineageFilter(
@@ -1361,13 +1501,16 @@ class LineageFilterFieldNumeric(LineageFilterField):
     def _get_filter(
         self, value: Union[int, float, date], operator: AtlanComparisonOperator
     ):
-        if isinstance(
-            value,
-            bool,  # needed because isinstance(value, int) evaluates to true when value is bool
-        ) or (
-            not isinstance(value, int)
-            and not isinstance(value, float)
-            and not isinstance(value, date)
+        if (
+            isinstance(
+                value,
+                bool,  # needed because isinstance(value, int) evaluates to true when value is bool
+            )
+            or (
+                not isinstance(value, int)
+                and not isinstance(value, float)
+                and not isinstance(value, date)
+            )
         ):
             raise ErrorCode.INVALID_PARAMETER_TYPE.exception_with_parameters(
                 type(value).__name__, "int, float or date"
