@@ -2,14 +2,31 @@ from __future__ import annotations
 
 import copy
 import dataclasses
-from typing import Optional, TypeVar, Union
+import logging
+from typing import TYPE_CHECKING, Dict, List, Optional, TypeVar, Union
 
 from pyatlan.client.asset import IndexSearchResults
+from pyatlan.errors import ErrorCode
 from pyatlan.model.aggregation import Aggregation
-from pyatlan.model.assets import Referenceable
+from pyatlan.model.assets import Referenceable, Tag
 from pyatlan.model.enums import EntityStatus
 from pyatlan.model.fields.atlan_fields import AtlanField
-from pyatlan.model.search import DSL, Bool, IndexSearchRequest, Query, SortItem, Term
+from pyatlan.model.search import (
+    DSL,
+    Bool,
+    IndexSearchRequest,
+    Query,
+    SortItem,
+    SpanNear,
+    SpanTerm,
+    SpanWithin,
+    Term,
+)
+
+if TYPE_CHECKING:
+    from pyatlan.client.atlan import AtlanClient
+
+LOGGER = logging.getLogger(__name__)
 
 SelfQuery = TypeVar("SelfQuery", bound="CompoundQuery")
 
@@ -20,9 +37,9 @@ class CompoundQuery:
     Class to compose compound queries combining various conditions.
     """
 
-    wheres: Optional[list[Query]] = None
-    where_nots: Optional[list[Query]] = None
-    where_somes: Optional[list[Query]] = None
+    wheres: Optional[List[Query]] = None
+    where_nots: Optional[List[Query]] = None
+    where_somes: Optional[List[Query]] = None
     _min_somes: int = 1
 
     @staticmethod
@@ -54,7 +71,7 @@ class CompoundQuery:
         return Referenceable.TYPE_NAME.eq(of.__name__)
 
     @staticmethod
-    def asset_types(one_of: list[type]) -> Query:
+    def asset_types(one_of: List[type]) -> Query:
         """
         Returns a query that will only match assets that are one of the types provided.
 
@@ -64,7 +81,7 @@ class CompoundQuery:
         return Referenceable.TYPE_NAME.within(list(map(lambda x: x.__name__, one_of)))
 
     @staticmethod
-    def super_types(one_of: Union[type, list[type]]) -> Query:
+    def super_types(one_of: Union[type, List[type]]) -> Query:
         """
         Returns a query that will match all assets that are a subtype of at least one of
         the types provided.
@@ -80,25 +97,30 @@ class CompoundQuery:
 
     @staticmethod
     def tagged(
-        with_one_of: Optional[list[str]] = None, directly: bool = False
+        client: AtlanClient,
+        with_one_of: Optional[List[str]] = None,
+        directly: bool = False,
     ) -> Query:
         """
         Returns a query that will only match assets that have at least one of the Atlan tags
         provided. This will match irrespective of the Atlan tag being directly applied to the
         asset, or if it was propagated to the asset.
 
+        :param client: connectivity to an Atlan tenant
         :param with_one_of: human-readable names of the Atlan tags
         :param directly: when True, the asset must have at least one Atlan tag directly assigned, otherwise
                          even propagated tags will suffice
         :returns: a query that will only match assets that have at least one of the Atlan tags provided
         """
-        from pyatlan.cache.atlan_tag_cache import AtlanTagCache
-
-        values: list[str] = []
+        values: List[str] = []
         if with_one_of:
             for name in with_one_of:
-                if tag_id := AtlanTagCache.get_id_for_name(name):
+                if tag_id := client.atlan_tag_cache.get_id_for_name(name):
                     values.append(tag_id)
+                else:
+                    raise ErrorCode.ATLAN_TAG_NOT_FOUND_BY_NAME.exception_with_parameters(
+                        name
+                    )
         if directly:
             if values:
                 return FluentSearch(
@@ -124,7 +146,100 @@ class CompoundQuery:
         ).to_query()
 
     @staticmethod
-    def assigned_term(qualified_names: Optional[list[str]] = None) -> Query:
+    def tagged_with_value(
+        client: AtlanClient,
+        atlan_tag_name: str,
+        value: str,
+        directly: bool = False,
+        source_tag_qualified_name: Optional[str] = None,
+    ) -> Query:
+        """
+        Returns a query that will match assets that have a
+        specific value for the specified tag (for source-synced tags).
+
+        :param client: connectivity to an Atlan tenant
+        :param atlan_tag_name: human-readable name of the Atlan tag
+        :param value: tag should have to match the query
+        :param directly: when `True`, the asset must have the tag and
+        value directly assigned (otherwise even propagated tags with the value will suffice)
+        :param source_tag_qualified_name: (optional) qualified name of
+        the source tag to match (when there are multiple)
+        :raises: AtlanError on any error communicating
+        with the API to refresh the Atlan tag cache
+        :returns: a query that will only match assets that have
+        a particular value assigned for the given Atlan tag
+        """
+        big_spans = []
+        little_spans = []
+        tag_id = client.atlan_tag_cache.get_id_for_name(atlan_tag_name) or ""
+        synced_tags = [
+            tag
+            for tag in (
+                FluentSearch()
+                .select()
+                .where(Tag.MAPPED_CLASSIFICATION_NAME.eq(tag_id))
+                .execute(client=client)
+            )
+        ]
+        if len(synced_tags) > 1 and source_tag_qualified_name is None:
+            synced_tag_qn = synced_tags[0].qualified_name or ""
+            LOGGER.warning(
+                "Multiple mapped source-synced tags found for tag %s -- using only the first: %s. "
+                "You can specify the `source_tag_qualified_name` so we can match to the specific one.",
+                atlan_tag_name,
+                synced_tag_qn,
+            )
+        elif synced_tags:
+            synced_tag_qn = (
+                source_tag_qualified_name or synced_tags[0].qualified_name or ""
+            )
+        else:
+            synced_tag_qn = "NON_EXISTENT"
+
+        # Contruct little spans
+        little_spans.append(
+            SpanTerm(field="__classificationsText.text", value="tagAttachmentValue")
+        )
+        for token in value.split(" "):
+            little_spans.append(
+                SpanTerm(field="__classificationsText.text", value=token)
+            )
+        little_spans.append(
+            SpanTerm(field="__classificationsText.text", value="tagAttachmentKey")
+        )
+
+        # Contruct big spans
+        big_spans.append(SpanTerm(field="__classificationsText.text", value=tag_id))
+        big_spans.append(
+            SpanTerm(field="__classificationsText.text", value=synced_tag_qn)
+        )
+
+        # Contruct final span query
+        span = SpanWithin(
+            little=SpanNear(clauses=little_spans, slop=0, in_order=True),
+            big=SpanNear(clauses=big_spans, slop=10000000, in_order=True),
+        )
+
+        # Without atlan tag propagation
+        if directly:
+            return (
+                FluentSearch()
+                .where(Referenceable.ATLAN_TAGS.eq(tag_id))
+                .where(span)
+                .to_query()
+            )
+        # With atlan tag propagation
+        return (
+            FluentSearch()
+            .where_some(Referenceable.ATLAN_TAGS.eq(tag_id))
+            .where_some(Referenceable.PROPAGATED_ATLAN_TAGS.eq(tag_id))
+            .min_somes(1)
+            .where(span)
+            .to_query()
+        )
+
+    @staticmethod
+    def assigned_term(qualified_names: Optional[List[str]] = None) -> Query:
         """
         Returns a query that will only match assets that have at least one term assigned.
         (If a list of qualified_names is specified, the assets that match must have at least
@@ -139,9 +254,9 @@ class CompoundQuery:
 
     def __init__(
         self,
-        wheres: Optional[list[Query]] = None,
-        where_nots: Optional[list[Query]] = None,
-        where_somes: Optional[list[Query]] = None,
+        wheres: Optional[List[Query]] = None,
+        where_nots: Optional[List[Query]] = None,
+        where_somes: Optional[List[Query]] = None,
         _min_somes: int = 1,
     ):
         self.wheres = wheres
@@ -231,11 +346,11 @@ class FluentSearch(CompoundQuery):
     Class to compose compound queries combining various conditions.
     """
 
-    sorts: Optional[list[SortItem]] = None
-    aggregations: Optional[dict[str, Aggregation]] = None
+    sorts: Optional[List[SortItem]] = None
+    aggregations: Optional[Dict[str, Aggregation]] = None
     _page_size: Optional[int] = None
-    _includes_on_results: Optional[list[str]] = None
-    _includes_on_relations: Optional[list[str]] = None
+    _includes_on_results: Optional[List[str]] = None
+    _includes_on_relations: Optional[List[str]] = None
 
     @classmethod
     def select(cls, include_archived=False) -> "FluentSearch":
@@ -257,15 +372,16 @@ class FluentSearch(CompoundQuery):
 
     def __init__(
         self,
-        wheres: Optional[list[Query]] = None,
-        where_nots: Optional[list[Query]] = None,
-        where_somes: Optional[list[Query]] = None,
+        wheres: Optional[List[Query]] = None,
+        where_nots: Optional[List[Query]] = None,
+        where_somes: Optional[List[Query]] = None,
         _min_somes: int = 1,
-        sorts: Optional[list[SortItem]] = None,
-        aggregations: Optional[dict[str, Aggregation]] = None,
+        sorts: Optional[List[SortItem]] = None,
+        aggregations: Optional[Dict[str, Aggregation]] = None,
         _page_size: Optional[int] = None,
-        _includes_on_results: Optional[list[str]] = None,
-        _includes_on_relations: Optional[list[str]] = None,
+        _includes_on_results: Optional[List[str]] = None,
+        _includes_on_relations: Optional[List[str]] = None,
+        _include_relationship_attributes: Optional[bool] = False,
     ):
         super().__init__(wheres, where_nots, where_somes, _min_somes)
         self.sorts = sorts
@@ -273,6 +389,7 @@ class FluentSearch(CompoundQuery):
         self._page_size = _page_size
         self._includes_on_results = _includes_on_results
         self._includes_on_relations = _includes_on_relations
+        self._include_relationship_attributes = _include_relationship_attributes
 
     def _clone(self) -> "FluentSearch":
         """
@@ -353,6 +470,22 @@ class FluentSearch(CompoundQuery):
             clone._includes_on_relations.append(field)
         return clone
 
+    def include_relationship_attributes(self, include: bool) -> "FluentSearch":
+        """
+        Add an attribute to include relationship attributes on each relationship in the results.
+
+        :param include: include relationship attributes on each relationship in the results
+        :returns: the fluent search with this parameter added
+        """
+        clone = self._clone()
+        # When enabling `include_relationship_attributes`
+        # it's mandatory to include the "name" field
+        # to ensure relationship names are included in the response.
+        # Omitting "name" will result in a 500 error from the metastore.
+        clone = self.include_on_relations("name")
+        clone._include_relationship_attributes = include
+        return clone
+
     def _dsl(self) -> DSL:
         """
         Translate the Atlan fluent search into an Atlan search DSL.
@@ -368,7 +501,8 @@ class FluentSearch(CompoundQuery):
         :returns: an Atlan search request that encapsulates the fluent search
         """
         dsl = self._dsl()
-        if self._page_size:
+        # Page size can be "0"
+        if self._page_size is not None:
             dsl.size = self._page_size
         if self.sorts:
             dsl.sort = self.sorts
@@ -379,6 +513,10 @@ class FluentSearch(CompoundQuery):
             request.attributes = self._includes_on_results
         if self._includes_on_relations:
             request.relation_attributes = self._includes_on_relations
+        if self._include_relationship_attributes:
+            request.include_relationship_attributes = (
+                self._include_relationship_attributes
+            )
         return request
 
     def count(self, client: AtlanClient) -> int:
@@ -394,14 +532,25 @@ class FluentSearch(CompoundQuery):
         request = IndexSearchRequest(dsl=dsl)
         return client.asset.search(request).count
 
-    def execute(self, client: AtlanClient) -> IndexSearchResults:
+    def execute(self, client: AtlanClient, bulk: bool = False) -> IndexSearchResults:
         """
         Run the fluent search to retrieve assets that match the supplied criteria.
+        `Note:` if the number of results exceeds the predefined threshold
+        (100,000 assets) this will be automatically converted into a `bulk` search.
 
-        :param client: client through which to retrieve the assets
+        :param client: client through which to retrieve the assets.
+        :param bulk: whether to run the search to retrieve assets that match the supplied criteria,
+        for large numbers of results (> `100,000`), defaults to `False`. Note: this will reorder the results
+        (based on creation timestamp) in order to iterate through a large number (more than `100,000`) results.
+        :raises InvalidRequestError:
+
+            - if bulk search is enabled (`bulk=True`) and any
+              user-specified sorting options are found in the search request.
+            - if bulk search is disabled (`bulk=False`) and the number of results
+              exceeds the predefined threshold (i.e: `100,000` assets)
+              and any user-specified sorting options are found in the search request.
+
+        :raises AtlanError: on any API communication issue
         :returns: an iterable list of assets that match the supplied criteria, lazily-fetched
         """
-        return client.asset.search(self.to_request())
-
-
-from pyatlan.client.atlan import AtlanClient  # noqa: E402
+        return client.asset.search(criteria=self.to_request(), bulk=bulk)
