@@ -4,14 +4,15 @@ from __future__ import annotations
 
 import copy
 from collections import deque
-from typing import TYPE_CHECKING, Any, Optional, Union
+from enum import Enum
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union
 
-from pydantic import Field, StrictBool, StrictInt, StrictStr, validate_arguments
+from pydantic.v1 import Field, StrictBool, StrictInt, StrictStr, validate_arguments
 
 if TYPE_CHECKING:
     from dataclasses import dataclass
 else:
-    from pydantic.dataclasses import dataclass
+    from pydantic.v1.dataclasses import dataclass
 
 from pyatlan.errors import ErrorCode
 from pyatlan.model.assets import Asset
@@ -40,13 +41,13 @@ class DirectedPair:
 
 @dataclass(frozen=True)
 class LineageGraph:
-    downstream_list: dict[str, dict[DirectedPair, None]]
-    upstream_list: dict[str, dict[DirectedPair, None]]
+    downstream_list: Dict[str, Dict[DirectedPair, None]]
+    upstream_list: Dict[str, Dict[DirectedPair, None]]
 
     @classmethod
-    def create(cls, relations: list[LineageRelation]) -> "LineageGraph":
-        downstream_list: dict[str, dict[DirectedPair, None]] = {}
-        upstream_list: dict[str, dict[DirectedPair, None]] = {}
+    def create(cls, relations: List[LineageRelation]) -> "LineageGraph":
+        downstream_list: Dict[str, Dict[DirectedPair, None]] = {}
+        upstream_list: Dict[str, Dict[DirectedPair, None]] = {}
 
         def add_relation(_relation: LineageRelation):
             if (
@@ -81,34 +82,34 @@ class LineageGraph:
 
     @staticmethod
     def get_asset_guids(
-        guid: str, guids: dict[str, dict[DirectedPair, None]]
-    ) -> list[str]:
+        guid: str, guids: Dict[str, Dict[DirectedPair, None]]
+    ) -> List[str]:
         if guid in guids:
             return list({pair.target_guid: None for pair in guids[guid].keys()}.keys())
         return []
 
     @staticmethod
     def get_process_guids(
-        guid: str, guids: dict[str, dict[DirectedPair, None]]
-    ) -> list[str]:
+        guid: str, guids: Dict[str, Dict[DirectedPair, None]]
+    ) -> List[str]:
         if guid in guids:
             return list({pair.process_guid: None for pair in guids[guid].keys()}.keys())
         return []
 
-    def get_downstream_asset_guids(self, guid: str) -> list[str]:
+    def get_downstream_asset_guids(self, guid: str) -> List[str]:
         return LineageGraph.get_asset_guids(guid, self.downstream_list)
 
-    def get_downstream_process_guids(self, guid: str) -> list[str]:
+    def get_downstream_process_guids(self, guid: str) -> List[str]:
         return LineageGraph.get_process_guids(guid, self.downstream_list)
 
-    def get_upstream_asset_guids(self, guid: str) -> list[str]:
+    def get_upstream_asset_guids(self, guid: str) -> List[str]:
         return LineageGraph.get_asset_guids(guid, self.upstream_list)
 
-    def get_upstream_process_guids(self, guid: str) -> list[str]:
+    def get_upstream_process_guids(self, guid: str) -> List[str]:
         return LineageGraph.get_process_guids(guid, self.upstream_list)
 
-    def get_all_downstream_asset_guids_dfs(self, guid: str) -> list[str]:
-        visited: dict[str, None] = {}
+    def get_all_downstream_asset_guids_dfs(self, guid: str) -> List[str]:
+        visited: Dict[str, None] = {}
         stack: deque[str] = deque()
         stack.append(guid)
         while len(stack) > 0:
@@ -120,8 +121,8 @@ class LineageGraph:
                         stack.append(downstream_guid)
         return list(visited.keys())
 
-    def get_all_upstream_asset_guids_dfs(self, guid: str) -> list[str]:
-        visited: dict[str, None] = {}
+    def get_all_upstream_asset_guids_dfs(self, guid: str) -> List[str]:
+        visited: Dict[str, None] = {}
         stack: deque[str] = deque()
         stack.append(guid)
         while len(stack) > 0:
@@ -142,9 +143,9 @@ class LineageResponse(AtlanObject):
     offset: int
     has_more_upstream_vertices: bool
     has_more_downstream_vertices: bool
-    guid_entity_map: dict[str, Asset]
-    relations: list[LineageRelation]
-    vertex_children_info: Optional[dict[str, Any]]
+    guid_entity_map: Dict[str, Asset]
+    relations: List[LineageRelation]
+    vertex_children_info: Optional[Dict[str, Any]]
     graph: Optional[LineageGraph] = None
 
     def get_graph(self):
@@ -154,12 +155,12 @@ class LineageResponse(AtlanObject):
 
     def get_all_downstream_asset_guids_dfs(
         self, guid: Optional[str] = None
-    ) -> list[str]:
+    ) -> List[str]:
         return self.get_graph().get_all_downstream_asset_guids_dfs(
             guid or self.base_entity_guid
         )
 
-    def get_all_downstream_assets_dfs(self, guid: Optional[str] = None) -> list[Asset]:
+    def get_all_downstream_assets_dfs(self, guid: Optional[str] = None) -> List[Asset]:
         return [
             self.guid_entity_map[guid]
             for guid in self.get_graph().get_all_downstream_asset_guids_dfs(
@@ -167,12 +168,12 @@ class LineageResponse(AtlanObject):
             )
         ]
 
-    def get_all_upstream_asset_guids_dfs(self, guid: Optional[str] = None) -> list[str]:
+    def get_all_upstream_asset_guids_dfs(self, guid: Optional[str] = None) -> List[str]:
         return self.get_graph().get_all_upstream_asset_guids_dfs(
             guid or self.base_entity_guid
         )
 
-    def get_all_upstream_assets_dfs(self, guid: Optional[str] = None) -> list[Asset]:
+    def get_all_upstream_assets_dfs(self, guid: Optional[str] = None) -> List[Asset]:
         return [
             self.guid_entity_map[guid]
             for guid in self.get_graph().get_all_upstream_asset_guids_dfs(
@@ -180,12 +181,12 @@ class LineageResponse(AtlanObject):
             )
         ]
 
-    def get_downstream_asset_guids(self, guid: Optional[str] = None) -> list[str]:
+    def get_downstream_asset_guids(self, guid: Optional[str] = None) -> List[str]:
         return self.get_graph().get_downstream_asset_guids(
             guid or self.base_entity_guid
         )
 
-    def get_downstream_assets(self, guid: Optional[str] = None) -> list[Asset]:
+    def get_downstream_assets(self, guid: Optional[str] = None) -> List[Asset]:
         return [
             self.guid_entity_map[guid]
             for guid in self.get_graph().get_downstream_asset_guids(
@@ -193,15 +194,15 @@ class LineageResponse(AtlanObject):
             )
         ]
 
-    def get_downstream_process_guids(self, guid: Optional[str] = None) -> list[str]:
+    def get_downstream_process_guids(self, guid: Optional[str] = None) -> List[str]:
         return self.get_graph().get_downstream_process_guids(
             guid or self.base_entity_guid
         )
 
-    def get_upstream_asset_guids(self, guid: Optional[str] = None) -> list[str]:
+    def get_upstream_asset_guids(self, guid: Optional[str] = None) -> List[str]:
         return self.get_graph().get_upstream_asset_guids(guid or self.base_entity_guid)
 
-    def get_upstream_assets(self, guid: Optional[str] = None) -> list[Asset]:
+    def get_upstream_assets(self, guid: Optional[str] = None) -> List[Asset]:
         return [
             self.guid_entity_map[guid]
             for guid in self.get_graph().get_upstream_asset_guids(
@@ -209,7 +210,7 @@ class LineageResponse(AtlanObject):
             )
         ]
 
-    def get_upstream_process_guids(self, guid: Optional[str] = None) -> list[str]:
+    def get_upstream_process_guids(self, guid: Optional[str] = None) -> List[str]:
         return self.get_graph().get_upstream_process_guids(
             guid or self.base_entity_guid
         )
@@ -228,8 +229,7 @@ class EntityFilter(AtlanObject):
         description="Name of the attribute on which filtering should be applied."
     )
     operator: AtlanComparisonOperator = Field(
-        description="Comparison that should be used when checking attribute_name"
-        " against the provided attribute_value."
+        description="Comparison that should be used when checking attribute_name against the provided attribute_value."
     )
     attribute_value: str = Field(
         description="Value that attribute_name should be compared against."
@@ -237,10 +237,15 @@ class EntityFilter(AtlanObject):
 
 
 class FilterList(AtlanObject):
-    condition: str = Field(
-        description="Whether the criteria must all match (AND) or any matching is sufficient (OR)."
+    class Condition(str, Enum):
+        AND = "AND"
+        OR = "OR"
+
+    condition: FilterList.Condition = Field(
+        default=Condition.AND,
+        description="Whether the criteria must all match (AND) or any matching is sufficient (OR).",
     )
-    criteria: list[EntityFilter] = Field(
+    criteria: List[EntityFilter] = Field(
         description="Basis on which to compare a result for inclusion.",
         alias="criterion",
     )
@@ -263,29 +268,43 @@ class LineageListRequest(SearchRequest):
         "Note that you cannot fetch both upstream and downstream at the same time."
     )
     entity_filters: Optional[FilterList] = Field(
-        description="Filters to apply on entities."
+        default=None, description="Filters to apply on entities."
     )
     entity_traversal_filters: Optional[FilterList] = Field(
+        default=None,
         description="Filters to apply for skipping traversal based on entities."
         "Any sub-graphs beyond the entities filtered out by these filters will not be included"
-        "in the lineage result."
+        "in the lineage result.",
+    )
+    relation_attributes: Optional[str] = Field(
+        default=None,
+        description="List of related attributes to be returned for each asset",
     )
     relationship_traversal_filters: Optional[FilterList] = Field(
+        default=None,
         description="Filters to apply for skipping traversal based on relationships."
         "Any sub-graphs beyond the relationships filtered out by these filters will not be included"
-        "in the lineage result."
+        "in the lineage result.",
     )
     offset: Optional[int] = Field(
-        description="Starting point for pagination.", alias="from"
+        default=None, description="Starting point for pagination.", alias="from"
     )
     size: Optional[int] = Field(
-        description="How many results to include in each page of results."
+        default=None, description="How many results to include in each page of results."
     )
     exclude_meanings: Optional[bool] = Field(
-        description="Whether to include assigned terms for assets (false) or not (true)."
+        default=None,
+        description="Whether to include assigned terms for assets (false) or not (true).",
     )
     exclude_classifications: Optional[bool] = Field(
-        description="Whether to include classifications for assets (false) or not (true)."
+        default=None,
+        description="Whether to include classifications for assets (false) or not (true).",
+    )
+    immediate_neighbors: Optional[bool] = Field(
+        default=None,
+        alias="immediateNeighbours",
+        description="Whether to include immediate neighbors of "
+        "the starting asset in the response (true), or not (false).",
     )
 
     @staticmethod
@@ -306,7 +325,7 @@ class LineageListRequest(SearchRequest):
             size=10,
             exclude_meanings=True,
             exclude_classifications=True,
-        )
+        )  # type: ignore[call-arg]
 
 
 class FluentLineage:
@@ -325,43 +344,73 @@ class FluentLineage:
         size: StrictInt = 10,
         exclude_meanings: StrictBool = True,
         exclude_atlan_tags: StrictBool = True,
-        includes_on_results: Optional[Union[list[AtlanField], AtlanField]] = None,
-        includes_in_results: Optional[Union[list[LineageFilter], LineageFilter]] = None,
-        where_assets: Optional[Union[list[LineageFilter], LineageFilter]] = None,
-        where_relationships: Optional[Union[list[LineageFilter], LineageFilter]] = None,
+        immediate_neighbors: StrictBool = False,
+        includes_on_results: Optional[
+            Union[List[str], str, List[AtlanField], AtlanField]
+        ] = None,
+        includes_in_results: Optional[Union[List[LineageFilter], LineageFilter]] = None,
+        includes_on_relations: Optional[
+            Union[List[str], str, List[AtlanField], AtlanField]
+        ] = None,
+        includes_condition: FilterList.Condition = FilterList.Condition.AND,
+        where_assets: Optional[Union[List[LineageFilter], LineageFilter]] = None,
+        assets_condition: FilterList.Condition = FilterList.Condition.AND,
+        where_relationships: Optional[Union[List[LineageFilter], LineageFilter]] = None,
+        relationships_condition: FilterList.Condition = FilterList.Condition.AND,
     ):
-        """Create a FluentLineage request.
+        """
+        Create a FluentLineage request.
+
         :param starting_guid: unique identifier (GUID) of the asset from which to start lineage
         :param depth: number of degrees of separation (hops) across which lineage should be fetched
         :param direction: direction of lineage to fetch (upstream or downstream)
         :param size: number of results to retrieve
         :param exclude_meanings: whether to include assigned terms for assets (False) or not (True)
         :param exclude_atlan_tags: whether to include Atlan tags for assets (False) or not (True)
+        :param immediate_neighbors: whether to include immediate neighbors
+        of the starting asset in the response (True), or not (False).
         :param includes_on_results: attributes to retrieve for each asset in the lineage results
-        :param includes_in_results: Assets to include in the results. Any assets not matching these filters will not
-        be included in the results, but will still be traversed in the lineage so that any assets beyond them are still
-        considered for inclusion in the results
-        :param where_assets: filters to apply on assets. Any assets excluded by the filters will exclude all assets
-        beyond, as well
-        :param where_relationships: filters to apply on relationships. Any relationships excluded by the filters will
-        exclude all assets and relationships beyond, as well
+        :param includes_in_results: assets to include in the results.
+            Any assets not matching these filters will not be included in the results,
+            but will still be traversed in the lineage so that any assets
+            beyond them are still considered for inclusion in the results.
+        :param includes_on_relations: attributes to retrieve for each asset related
+        to the assets in the results (for internal use, unchecked!).
+        :param includes_condition: whether the `includes_in_results` criteria
+        should be combined (AND) or any matching is sufficient (OR)
+        :param where_assets: filters to apply on assets. Any assets excluded
+        by the filters will exclude all assets beyond, as well
+        :param assets_condition: whether the `where_assets` criteria
+        should be combined (AND) or any matching is sufficient (OR)
+        :param where_relationships: filters to apply on relationships.
+        Any relationships excluded by the filters will exclude all assets and relationships beyond, as well
+        :param relationships_condition: whether the `where_relationships` criteria
+        should be combined (AND) or any matching is sufficient (OR)
         """
 
         self._depth: int = depth
         self._direction: LineageDirection = direction
         self._exclude_atlan_tags: bool = exclude_atlan_tags
         self._exclude_meanings: bool = exclude_meanings
-
-        self._includes_on_results: list[AtlanField] = self._to_list(includes_on_results)
-        self._includes_in_results: list[LineageFilter] = self._to_list(
+        self._immediate_neighbors: bool = immediate_neighbors
+        self._includes_on_results: List[Union[str, AtlanField]] = self._to_list(
+            includes_on_results
+        )
+        self._includes_in_results: List[LineageFilter] = self._to_list(
             includes_in_results
         )
+        self._includes_on_relations: List[Union[str, AtlanField]] = self._to_list(
+            includes_on_relations
+        )
+        self._includes_condition: FilterList.Condition = includes_condition
         self._size: int = size
         self._starting_guid = starting_guid
-        self._where_assets: list[LineageFilter] = self._to_list(where_assets)
-        self._where_relationships: list[LineageFilter] = self._to_list(
+        self._where_assets: List[LineageFilter] = self._to_list(where_assets)
+        self._assets_condition: FilterList.Condition = assets_condition
+        self._where_relationships: List[LineageFilter] = self._to_list(
             where_relationships
         )
+        self._relationships_condition: FilterList.Condition = relationships_condition
 
     @staticmethod
     def _to_list(value):
@@ -376,34 +425,45 @@ class FluentLineage:
         return copy.deepcopy(self)
 
     def depth(self, depth: StrictInt) -> "FluentLineage":
-        """Adds the depth to traverse the lineage.
+        """
+        Adds the depth to traverse the lineage.
+
         :param depth: number of degrees of separation (hops) across which lineage should be fetched
-        :returns: the FluentLineage with this depth criterion added"""
+        :returns: the FluentLineage with this depth criterion added
+        """
         validate_type(name="depth", _type=int, value=depth)
         clone = self._clone()
         clone._depth = depth
         return clone
 
     def direction(self, direction: LineageDirection) -> "FluentLineage":
-        """Adds the direction to traverse the lineage.
+        """
+        Adds the direction to traverse the lineage.
+
         :param direction: direction of lineage to fetch (upstream or downstream)
-        :returns: the FluentLineage with this direction criterion added"""
+        :returns: the FluentLineage with this direction criterion added
+        """
         validate_type(name="direction", _type=LineageDirection, value=direction)
         clone = self._clone()
         clone._direction = direction
         return clone
 
     def size(self, size: StrictInt) -> "FluentLineage":
-        """Adds the size to traverse the lineage.
+        """
+        Adds the size to traverse the lineage.
+
         :param size: number of results to retrieve
-        :returns: the FluentLineage with this size criterion added"""
+        :returns: the FluentLineage with this size criterion added
+        """
         validate_type(name="size", _type=int, value=size)
         clone = self._clone()
         clone._size = size
         return clone
 
     def exclude_atlan_tags(self, exclude_atlan_tags: StrictBool) -> "FluentLineage":
-        """Adds the exclude_atlan_tags to traverse the lineage.
+        """
+        Adds the exclude_atlan_tags to traverse the lineage.
+
         :param exclude_atlan_tags: whether to include Atlan tags for assets (False) or not (True)
         :returns: the FluentLineage with this exclude_atlan_tags criterion added
         """
@@ -413,19 +473,38 @@ class FluentLineage:
         return clone
 
     def exclude_meanings(self, exclude_meanings: StrictBool) -> "FluentLineage":
-        """Adds the exclude_meanings to traverse the lineage.
+        """
+        Adds the exclude_meanings to traverse the lineage.
+
         :param exclude_meanings: whether to include assigned terms for assets (False) or not (True)
-        :returns: the FluentLineage with this exclude_meanings criterion added"""
+        :returns: the FluentLineage with this exclude_meanings criterion added
+        """
         validate_type(name="exclude_meanings", _type=bool, value=exclude_meanings)
         clone = self._clone()
         clone._exclude_meanings = exclude_meanings
         return clone
 
-    def include_on_results(self, field: AtlanField) -> "FluentLineage":
-        """Adds the include_on_results to traverse the lineage.
+    def immediate_neighbors(self, immediate_neighbors: StrictBool) -> "FluentLineage":
+        """
+        Adds the immediate_neighbors to traverse the lineage.
+
+        :param immediate_neighbors: whether to include immediate neighbors
+        of the starting asset in the response (True), or not (False)
+        :returns: the FluentLineage with this immediate_neighbors criterion added
+        """
+        validate_type(name="immediate_neighbors", _type=bool, value=immediate_neighbors)
+        clone = self._clone()
+        clone._immediate_neighbors = immediate_neighbors
+        return clone
+
+    def include_on_results(self, field: Union[str, AtlanField]) -> "FluentLineage":
+        """
+        Adds the include_on_results to traverse the lineage.
+
         :param field: attributes to retrieve for each asset in the lineage results
-        :returns: the FluentLineage with this include_on_results criterion added"""
-        validate_type(name="field", _type=AtlanField, value=field)
+        :returns: the FluentLineage with this include_on_results criterion added
+        """
+        validate_type(name="field", _type=(str, AtlanField), value=field)
         clone = self._clone()
         clone._includes_on_results.append(field)
         return clone
@@ -433,9 +512,12 @@ class FluentLineage:
     def include_in_results(self, lineage_filter: LineageFilter) -> "FluentLineage":
         """
         Adds the include_on_results to traverse the lineage.
-        :param lineage_filter: Assets to include in the results. Any assets not matching this filters will not be
-        included in the results, but will still be traversed in the lineage so that any assets beyond them are still
-        considered for inclusion in the results
+
+        :param lineage_filter: Assets to include in the results.
+            Any assets not matching this filters will not be
+            included in the results, but will still be traversed
+            in the lineage so that any assets beyond them are still
+            considered for inclusion in the results
         :returns: the FluentLineage with this include_in_results criterion added
         """
         validate_type(name="lineage_filter", _type=LineageFilter, value=lineage_filter)
@@ -443,11 +525,44 @@ class FluentLineage:
         clone._includes_in_results.append(lineage_filter)
         return clone
 
+    def include_on_relations(self, field: Union[str, AtlanField]) -> "FluentLineage":
+        """
+        Adds the include_on_relations to traverse the lineage.
+
+        :param field: attributes to retrieve for each asset related
+        to the assets in the results (for internal use, unchecked!)
+        :returns: the FluentLineage with this include_on_relations criterion added
+        """
+        validate_type(name="field", _type=(str, AtlanField), value=field)
+        clone = self._clone()
+        clone._includes_on_relations.append(field)
+        return clone
+
+    def includes_condition(
+        self, includes_condition: FilterList.Condition
+    ) -> "FluentLineage":
+        """
+        Adds the filter condition to `include_on_results`.
+
+        :param includes_condition: whether the `includes_in_results`
+        criteria should be combined (AND) or any matching is sufficient (OR)
+        :returns: the FluentLineage with this includes_condition criterion added
+        """
+        validate_type(
+            name="includes_condition",
+            _type=FilterList.Condition,
+            value=includes_condition,
+        )
+        clone = self._clone()
+        clone._includes_condition = includes_condition
+        return clone
+
     def where_assets(self, lineage_filter: LineageFilter) -> "FluentLineage":
         """
         Adds a filters to apply on assets.
-        :param lineage_filter: a filter to apply on assets. Any assets excluded by the filters will exclude all
-        assets beyond, as well
+
+        :param lineage_filter: a filter to apply on assets.
+        Any assets excluded by the filters will exclude all assets beyond, as well
         :returns: the FluentLineage with this where_assets criterion added
         """
         validate_type(name="lineage_filter", _type=LineageFilter, value=lineage_filter)
@@ -455,20 +570,62 @@ class FluentLineage:
         clone._where_assets.append(lineage_filter)
         return clone
 
+    def assets_condition(
+        self, assets_condition: FilterList.Condition
+    ) -> "FluentLineage":
+        """
+        Adds the filter condition to `where_assets`.
+
+        :param assets_condition: whether the `where_assets`
+        criteria should be combined (AND) or any matching is sufficient (OR)
+        :returns: the FluentLineage with this assets_condition criterion added
+        """
+        validate_type(
+            name="assets_condition",
+            _type=FilterList.Condition,
+            value=assets_condition,
+        )
+        clone = self._clone()
+        clone._assets_condition = assets_condition
+        return clone
+
     def where_relationships(self, lineage_filter: LineageFilter) -> "FluentLineage":
-        """Filters to apply on relationships.
-        :param lineage_filter: any relationships excluded by the filter will exclude all assets and
-        relationships beyond, as well.
-        :returns: the FluentLineage with this where_relationships criterion added"""
+        """
+        Filters to apply on relationships.
+
+        :param lineage_filter: any relationships excluded
+        by the filter will exclude all assets and relationships beyond, as well.
+        :returns: the FluentLineage with this where_relationships criterion added
+        """
         validate_type(name="lineage_filter", _type=LineageFilter, value=lineage_filter)
         clone = self._clone()
         clone._where_relationships.append(lineage_filter)
         return clone
 
+    def relationships_condition(
+        self, relationships_condition: FilterList.Condition
+    ) -> "FluentLineage":
+        """
+        Adds the filter condition to `where_relationships`.
+
+        :param relationships_condition: whether the `where_relationships`
+        criteria should be combined (AND) or any matching is sufficient (OR)
+        :returns: the FluentLineage with this relationships_condition criterion added
+        """
+        validate_type(
+            name="relationships_condition",
+            _type=FilterList.Condition,
+            value=relationships_condition,
+        )
+        clone = self._clone()
+        clone._relationships_condition = relationships_condition
+        return clone
+
     @property
     def request(self) -> LineageListRequest:
         """
-        :returns: the LineageListRequest that encapsulates information specified in this FluentLineage
+        :returns: the LineageListRequest that encapsulates
+        information specified in this FluentLineage
         """
         request = LineageListRequest.create(guid=self._starting_guid)
         if self._depth:
@@ -479,6 +636,8 @@ class FluentLineage:
             request.exclude_classifications = self._exclude_atlan_tags
         if self._exclude_meanings is not None:
             request.exclude_meanings = self._exclude_meanings
+        if self._immediate_neighbors is not None:
+            request.immediate_neighbors = self._immediate_neighbors
         if self._includes_in_results:
             criteria = [
                 EntityFilter(
@@ -488,10 +647,18 @@ class FluentLineage:
                 )
                 for _filter in self._includes_in_results
             ]
-            request.entity_filters = FilterList(condition="AND", criteria=criteria)
+            request.entity_filters = FilterList(
+                condition=self._includes_condition, criteria=criteria
+            )  # type: ignore
         if self._includes_on_results:
             request.attributes = [
-                field.atlan_field_name for field in self._includes_on_results
+                field.atlan_field_name if isinstance(field, AtlanField) else field
+                for field in self._includes_on_results
+            ]
+        if self._includes_on_relations:
+            request.relation_attributes = [  # type: ignore
+                field.atlan_field_name if isinstance(field, AtlanField) else field
+                for field in self._includes_on_relations
             ]
         if self._size:
             request.size = self._size
@@ -505,8 +672,8 @@ class FluentLineage:
                 for _filter in self._where_assets
             ]
             request.entity_traversal_filters = FilterList(
-                condition="AND", criteria=criteria
-            )
+                condition=self._assets_condition, criteria=criteria
+            )  # type: ignore[call-arg]
         if self._where_relationships:
             criteria = [
                 EntityFilter(
@@ -517,6 +684,6 @@ class FluentLineage:
                 for _filter in self._where_relationships
             ]
             request.relationship_traversal_filters = FilterList(
-                condition="AND", criteria=criteria
-            )
+                condition=self._relationships_condition, criteria=criteria
+            )  # type: ignore[call-arg]
         return request
