@@ -1,15 +1,16 @@
 # SPDX-License-Identifier: Apache-2.0
 # Copyright 2022 Atlan Pte. Ltd.
 # Based on original code from https://github.com/elastic/elasticsearch-dsl-py.git (under Apache-2.0 license)
+from __future__ import annotations
+
 import copy
 from abc import ABC, abstractmethod
 from datetime import datetime
 from enum import Enum
 from itertools import chain
-from json import dumps, loads
-from typing import Any, Literal, Optional, Union
+from typing import Any, Dict, List, Literal, Optional, Sequence, Union
 
-from pydantic import (
+from pydantic.v1 import (
     ConfigDict,
     Field,
     StrictBool,
@@ -20,8 +21,8 @@ from pydantic import (
     validate_arguments,
     validator,
 )
-from pydantic.config import Extra
-from pydantic.dataclasses import dataclass
+from pydantic.v1.config import Extra
+from pydantic.v1.dataclasses import dataclass
 
 from pyatlan.model.aggregation import Aggregation
 from pyatlan.model.core import AtlanObject, SearchRequest
@@ -30,6 +31,7 @@ from pyatlan.model.enums import (
     CertificateStatus,
     ChildScoreMode,
     SortOrder,
+    UTMTags,
 )
 
 SearchFieldType = Union[StrictStr, StrictInt, StrictFloat, StrictBool, datetime]
@@ -123,8 +125,7 @@ class Query(ABC):
         return copy.deepcopy(self)
 
     @abstractmethod
-    def to_dict(self) -> dict[Any, Any]:
-        ...
+    def to_dict(self) -> Dict[Any, Any]: ...
 
 
 @dataclass(config=ConfigDict(smart_union=True, extra="forbid"))  # type: ignore
@@ -145,7 +146,7 @@ class MatchAll(Query):
     def __invert__(self):
         return MatchNone()
 
-    def to_dict(self) -> dict[Any, Any]:
+    def to_dict(self) -> Dict[Any, Any]:
         value = {"boost": self.boost} if self.boost else {}
         return {self.type_name: value}
 
@@ -170,7 +171,7 @@ class MatchNone(Query):
     def __invert__(self):
         return MatchAll()
 
-    def to_dict(self) -> dict[Any, Any]:
+    def to_dict(self) -> Dict[Any, Any]:
         return {"match_none": {}}
 
 
@@ -181,10 +182,10 @@ class Exists(Query):
 
     @classmethod
     @validate_arguments()
-    def with_custom_metadata(cls, set_name: StrictStr, attr_name: StrictStr):
-        from pyatlan.cache.custom_metadata_cache import CustomMetadataCache
-
-        if attr_id := CustomMetadataCache.get_attr_id_for_name(
+    def with_custom_metadata(
+        cls, client: Any, set_name: StrictStr, attr_name: StrictStr
+    ):
+        if attr_id := client.custom_metadata_cache.get_attr_id_for_name(
             set_name=set_name, attr_name=attr_name
         ):
             return cls(field=attr_id)
@@ -367,11 +368,13 @@ class Term(Query):
     @classmethod
     @validate_arguments()
     def with_custom_metadata(
-        cls, set_name: StrictStr, attr_name: StrictStr, value: SearchFieldType
+        cls,
+        client: Any,
+        set_name: StrictStr,
+        attr_name: StrictStr,
+        value: SearchFieldType,
     ):
-        from pyatlan.cache.custom_metadata_cache import CustomMetadataCache
-
-        if attr_id := CustomMetadataCache.get_attr_id_for_name(
+        if attr_id := client.custom_metadata_cache.get_attr_id_for_name(
             set_name=set_name, attr_name=attr_name
         ):
             return cls(field=attr_id, value=value)
@@ -398,7 +401,8 @@ class Term(Query):
     @classmethod
     @validate_arguments()
     def with_glossary(
-        cls, qualified_name: constr(strip_whitespace=True, min_length=1, strict=True)  # type: ignore
+        cls,
+        qualified_name: constr(strip_whitespace=True, min_length=1, strict=True),  # type: ignore
     ):
         return cls(field=TermAttributes.GLOSSARY.value, value=qualified_name)
 
@@ -494,13 +498,13 @@ class Term(Query):
 @dataclass
 class Terms(Query):
     field: str
-    values: list[str]
+    values: List[str]
     boost: Optional[float] = None
     type_name: Literal["terms"] = "terms"
 
     @classmethod
     @validate_arguments()
-    def with_type_name(cls, values: list[str]):
+    def with_type_name(cls, values: List[str]):
         return cls(field=TermAttributes.TYPE_NAME.value, values=values)
 
     def to_dict(self):
@@ -510,12 +514,77 @@ class Terms(Query):
         return {self.type_name: terms}
 
 
+@dataclass
+class SpanWithin(Query):
+    little: Optional[Query] = None
+    big: Optional[Query] = None
+    boost: Optional[float] = None
+    type_name: Literal["span_within"] = "span_within"
+
+    def to_dict(self):
+        span_within = {}
+        if self.little is not None:
+            span_within["little"] = self.little
+        if self.big is not None:
+            span_within["big"] = self.big
+        if self.boost is not None:
+            span_within["boost"] = self.boost
+        return {self.type_name: span_within}
+
+
+@dataclass
+class SpanTerm(Query):
+    field: str
+    value: SearchFieldType
+    boost: Optional[float] = None
+    type_name: Literal["span_term"] = "span_term"
+
+    def to_dict(self):
+        span_term = {self.field: self.value}
+        if self.boost is not None:
+            span_term["boost"] = self.boost
+        return {self.type_name: span_term}
+
+
+@dataclass
+class SpanNear(Query):
+    clauses: Optional[Sequence[Query]] = None
+    in_order: Optional[bool] = None
+    slop: Optional[int] = None
+    type_name: Literal["span_near"] = "span_near"
+
+    def to_dict(self):
+        span_near = {}
+        if self.clauses is not None:
+            span_near["clauses"] = self.clauses
+        if self.in_order is not None:
+            span_near["in_order"] = self.in_order
+        if self.slop is not None:
+            span_near["slop"] = self.slop
+        return {self.type_name: span_near}
+
+
+@dataclass
+class Span(Query):
+    span_within: Optional[Query] = None
+    span_near: Optional[Query] = None
+    type_name: Literal["span"] = "span"
+
+    def to_dict(self):
+        span = {}
+        if self.span_within is not None:
+            span["span_within"] = self.span_within
+        if self.span_near is not None:
+            span["span_near"] = self.span_near
+        return {self.type_name: span}
+
+
 @dataclass(config=ConfigDict(smart_union=True, extra="forbid"))  # type: ignore
 class Bool(Query):
-    must: list[Query] = Field(default_factory=list)
-    should: list[Query] = Field(default_factory=list)
-    must_not: list[Query] = Field(default_factory=list)
-    filter: list[Query] = Field(default_factory=list)
+    must: List[Query] = Field(default_factory=list)
+    should: List[Query] = Field(default_factory=list)
+    must_not: List[Query] = Field(default_factory=list)
+    filter: List[Query] = Field(default_factory=list)
     type_name: Literal["bool"] = "bool"
     boost: Optional[float] = None
     minimum_should_match: Optional[int] = None
@@ -619,7 +688,7 @@ class Bool(Query):
 
     __rand__ = __and__
 
-    def to_dict(self) -> dict[Any, Any]:
+    def to_dict(self) -> Dict[Any, Any]:
         clauses = {}
 
         def add_clause(name):
@@ -716,11 +785,13 @@ class Prefix(Query):
     def with_type_name(cls, value: StrictStr):
         return cls(field=TermAttributes.TYPE_NAME.value, value=value)
 
-    def to_dict(self) -> dict[Any, Any]:
-        parameters: dict[str, Any] = {
-            "value": int(self.value.timestamp() * 1000)
-            if isinstance(self.value, datetime)
-            else self.value
+    def to_dict(self) -> Dict[Any, Any]:
+        parameters: Dict[str, Any] = {
+            "value": (
+                int(self.value.timestamp() * 1000)
+                if isinstance(self.value, datetime)
+                else self.value
+            )
         }
 
         if self.case_insensitive is not None:
@@ -868,12 +939,12 @@ class Range(Query):
             time_zone=time_zone,
         )
 
-    def to_dict(self) -> dict[Any, Any]:
+    def to_dict(self) -> Dict[Any, Any]:
         def get_value(attribute_name):
             if hasattr(self, attribute_name):
                 attribute_value = getattr(self, attribute_name)
                 if isinstance(attribute_value, datetime):
-                    attribute_value = int(self.value.timestamp() * 1000)
+                    attribute_value = int(attribute_value.timestamp() * 1000)
             else:
                 attribute_value = None
             return attribute_value
@@ -1752,9 +1823,9 @@ class Match(Query):
         if self.analyzer is not None:
             parameters["analyzer"] = self.analyzer
         if self.auto_generate_synonyms_phrase_query is not None:
-            parameters[
-                "auto_generate_synonyms_phrase_query"
-            ] = self.auto_generate_synonyms_phrase_query
+            parameters["auto_generate_synonyms_phrase_query"] = (
+                self.auto_generate_synonyms_phrase_query
+            )
         if self.fuzziness is not None:
             parameters["fuzziness"] = self.fuzziness
         if self.fuzzy_transpositions is not None:
@@ -1776,6 +1847,29 @@ class Match(Query):
         return {self.type_name: {self.field: parameters}}
 
 
+@dataclass(config=ConfigDict(smart_union=True, extra="forbid"))  # type: ignore
+class MatchPhrase(Query):
+    field: str
+    query: StrictStr
+    analyzer: Optional[str] = None
+    slop: Optional[int] = None
+    zero_terms_query: Optional[Literal["none", "all"]] = None
+    boost: Optional[float] = None
+    type_name: Literal["match_phrase"] = "match_phrase"
+
+    def to_dict(self):
+        parameters = {"query": self.query}
+        if self.analyzer is not None:
+            parameters["analyzer"] = self.analyzer
+        if self.slop is not None:
+            parameters["slop"] = self.slop
+        if self.zero_terms_query is not None:
+            parameters["zero_terms_query"] = self.zero_terms_query
+        if self.boost is not None:
+            parameters["boost"] = self.boost
+        return {self.type_name: {self.field: parameters}}
+
+
 @dataclass(config=ConfigDict(smart_union=True, extra="forbid"))  # type: ignore
 class SortItem:
     field: StrictStr
@@ -1788,6 +1882,15 @@ class SortItem:
             parameters["nested"] = {"path": self.nested_path}
         return {self.field: parameters}
 
+    @classmethod
+    def from_dict(cls, d: Dict[str, Any]) -> "SortItem":
+        field, params = list(d.items())[0]
+
+        order = params.get("order", SortOrder.ASCENDING)
+        nested_path = params.get("nested", {}).get("path", None)
+
+        return cls(field=field, order=order, nested_path=nested_path)
+
     @validator("order", always=True)
     def validate_order(cls, v, values):
         if not v and "field" in values:
@@ -1796,15 +1899,16 @@ class SortItem:
 
 
 class DSL(AtlanObject):
-    from_: int = Field(0, alias="from")
-    size: int = 100
-    aggregations: dict[str, Aggregation] = Field(default_factory=dict)
-    track_total_hits: bool = Field(True, alias="track_total_hits")
-    post_filter: Optional[Query] = Field(alias="post_filter")
-    query: Optional[Query]
-    sort: list[SortItem] = Field(
-        alias="sort", default=[SortItem(TermAttributes.GUID.value)]
+    from_: int = Field(default=0, alias="from")
+    size: int = Field(default=300)
+    aggregations: Dict[str, Aggregation] = Field(default_factory=dict)
+    track_total_hits: Optional[bool] = Field(default=True, alias="track_total_hits")
+    post_filter: Optional[Union[Dict[str, Any], Query]] = Field(
+        default=None, alias="post_filter"
     )
+    query: Optional[Union[Dict[str, Any], Query]]
+    req_class_name: Optional[str] = Field(default=None, exclude=True)
+    sort: List[SortItem] = Field(default_factory=list, alias="sort")
 
     class Config:
         json_encoders = {Query: lambda v: v.to_dict(), SortItem: lambda v: v.to_dict()}
@@ -1812,7 +1916,13 @@ class DSL(AtlanObject):
     def __init__(__pydantic_self__, **data: Any) -> None:
         super().__init__(**data)
         __pydantic_self__.__fields_set__.update(
-            ["from_", "size", "track_total_hits", "sort", "aggregations"]
+            [
+                "from_",
+                "size",
+                "track_total_hits",
+                "sort",
+                "aggregations",
+            ]
         )
 
     @validator("query", always=True)
@@ -1822,41 +1932,127 @@ class DSL(AtlanObject):
         else:
             raise ValueError("Either query or post_filter is required")
 
+    @validator("sort", pre=True)
+    def sortitem_from_dict(cls, sort):
+        """
+        Convert dictionary entries into SortItem instances before validation
+        to ensure proper deserialization and prevent validation issues.
+        """
+        if all(isinstance(item, dict) for item in sort):
+            sort = [SortItem.from_dict(item) for item in sort]
+        return sort
+
+    @validator("sort", always=True)
+    def validate_sort(cls, sort, values):
+        missing_guid_sort = True
+        sort_by_guid = "__guid"
+        auditsearch_sort_by_guid = "entityId"
+        searchlog_sort_by_guid = "entityGuidsAll"
+        req_class_name = values.get("req_class_name")
+        # Check if the sort by GUID is included
+        for option in sort:
+            if option.field and option.field in (
+                sort_by_guid,
+                auditsearch_sort_by_guid,
+                searchlog_sort_by_guid,
+            ):
+                missing_guid_sort = False
+                break
+        if missing_guid_sort:
+            if req_class_name == "SearchLogRequest":
+                sort.append(SortItem(searchlog_sort_by_guid))
+            elif req_class_name == "AuditSearchRequest":
+                sort.append(SortItem(auditsearch_sort_by_guid))
+            elif req_class_name == "IndexSearchRequest":
+                sort.append(SortItem(sort_by_guid))
+        return sort
+
+
+class IndexSearchRequestMetadata(AtlanObject):
+    save_search_log: bool = Field(
+        default=False,
+        description="Whether to log this search (True) or not (False)",
+    )
+    utm_tags: List[str] = Field(
+        default_factory=list,
+        description="Tags to associate with the search request",
+    )
+
 
 class IndexSearchRequest(SearchRequest):
     dsl: DSL
-    relation_attributes: list[str] = Field(
+    relation_attributes: Optional[List[str]] = Field(
         default_factory=list, alias="relationAttributes"
     )
-    suppress_logs: Optional[bool] = Field(alias="suppressLogs")
+    suppress_logs: Optional[bool] = Field(default=None, alias="suppressLogs")
     show_search_score: Optional[bool] = Field(
+        default=None,
         description="When true, include the score of each result. By default, this is false and scores are excluded.",
         alias="showSearchScore",
     )
     exclude_meanings: Optional[bool] = Field(
+        default=None,
         description="Whether to include term relationships for assets (false) or not (true). By default, this is false "
         "and term relationships are therefore included.",
         alias="excludeMeanings",
     )
     exclude_atlan_tags: Optional[bool] = Field(
+        default=None,
         description="Whether to include Atlan tags for assets (false) or not (true). By default, this is false and "
         "Atlan tags are therefore included.",
         alias="excludeClassifications",
     )
     allow_deleted_relations: Optional[bool] = Field(
+        default=None,
         description="Whether to include deleted relationships to this asset (true) or not (false). By default, "
         "this is false and therefore only active (not deleted) relationships will be included.",
         alias="allowDeletedRelations",
     )
-
-    def get_dsl_str(self) -> str:
-        dsl_json_str = self.json(by_alias=True, exclude_none=True)
-        dsl_query_dict = dict(query=loads(dsl_json_str))
-        return dumps(dsl_query_dict)
+    include_atlan_tag_names: Optional[bool] = Field(
+        default=None,
+        description=(
+            "Whether to include Atlan tag names for this asset (`True`) or not (`False`) "
+            "Note: This will only work when `exclude_atlan_tags` is set to `True`."
+        ),
+        alias="includeClassificationNames",
+    )
+    persona: Optional[str] = Field(
+        default=None,
+        description="Qualified name of the persona eg: default/tL6uqsrZGuf1hz9XFYnw8x",
+    )
+    purpose: Optional[str] = Field(
+        default=None,
+        description="Qualified name of the purpose eg: default/zL6uqsrZGuf1hz9XFYnw9x",
+    )
+    include_relationship_attributes: Optional[bool] = Field(
+        default=False,
+        description=(
+            "Whether to include relationship-level attributes "
+            "for any relationships to each asset (True) or not (False). "
+            "By default, this is `False` and therefore "
+            "relationship-level attributes are not included."
+        ),
+    )
+    request_metadata: Optional[IndexSearchRequestMetadata] = Field(
+        default_factory=lambda: IndexSearchRequestMetadata(
+            # Set this to `False` to prevent the frequent
+            # Out of memory (OOM) issue in Metastore pods.
+            save_search_log=False,
+            utm_tags=[UTMTags.PROJECT_SDK_PYTHON],
+        )
+    )
 
     class Config:
         json_encoders = {Query: lambda v: v.to_dict(), SortItem: lambda v: v.to_dict()}
 
+    def __init__(__pydantic_self__, **data: Any) -> None:
+        dsl = data.get("dsl")
+        class_name = __pydantic_self__.__class__.__name__
+        if dsl and isinstance(dsl, DSL) and not dsl.req_class_name:
+            data["dsl"] = DSL(req_class_name=class_name, **dsl.dict(exclude_unset=True))
+        super().__init__(**data)
+        __pydantic_self__.__fields_set__.update(["request_metadata"])
+
 
 def with_active_glossary(name: StrictStr) -> "Bool":
     return (
